      SUBROUTINE EIGEN(MM, M, N, A, DMEIG, EIGVC, EIGVL, WORK, IERR,
     *                 OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      COMPUTES THE EIGENVECTORS OF THE MATRIX A'A
C
C   DESCRIPTION
C   -----------
C
C   1.  THE MATRIX A'A IS CALCULATED AND THE LINPACK SUBROUTINE SSIEV IS
C       CALLED TO COMPUTE ITS EIGENVALUES AND EIGENVECTORS.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C   DMEIG INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX EIGVC.  MUST BE AT LEAST N.
C
C   WORK  REAL VECTOR DIMENSIONED AT LEAST 2*N.
C         WORK VECTOR (FOR THE EIGENVECTOR ROUTINE).
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR ERROR MESSAGES.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   EIGVC REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMEIG AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST N.
C         MATRIX WHOSE COLUMNS ARE THE EIGENVECTORS.
C
C   EIGVL REAL VECTOR DIMENSIONED AT LEAST N.
C         VECTOR WHICH STORES THE EIGENVALUES.
C
C   IERR  INTEGER SCALAR.
C         ERROR FLAG.
C
C         IERR = 0, NO ERROR WAS DETECTED DURING EXECUTION.
C
C         IERR = K, THE EIGENVECTOR ROUTINE FAILED TO CONVERGE.
C                   EIGENVALUES 1, ..., K-1 ARE CORRECT.  ERROR
C                   MESSAGE FROM CMLIB ROUTINE SSIEV.
C
C   REFERENCES
C   ----------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 46.
C
C     NBS CORE MATH LIBRARY, VOLS. 1-4 (GAITHERSBURG: QA297.C69 IN NBS
C     LIBRARY, ADMIN E-120).
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMEIG, OUNIT
      DIMENSION A(MM,*), EIGVC(DMEIG,*), EIGVL(*), WORK(*)
C
C     COMPUTE A'A
C
      DO 20 I = 1 , N
         DO 20 J = I , N
            EIGVC(I,J) = 0.
            DO 10 K = 1 , M
  10           EIGVC(I,J) = EIGVC(I,J) + A(K,I) * A(K,J)
  20  CONTINUE
C
      JOB = 1
      CALL SSIEV(EIGVC, DMEIG, N, EIGVL, WORK, JOB, IERR)
      IF (IERR .NE. 0) THEN
         IF (OUNIT .GT. 0) WRITE(OUNIT,7) IERR
  7      FORMAT(1X, 'ERROR NUMBER ', I3, 'IN COMPUTING EIGENVALUES ')
         RETURN
      ENDIF
      RETURN
      END
