      SUBROUTINE HIST(NPTS, X, XMIN, XMAX, NI, P, TITLE, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      PRINTS A UNIVARIATE HISTOGRAM
C
C   DESCRIPTION
C   -----------
C
C   1.  EACH VALUE IS INITIALLY MAPPED INTO ONE OF 100 BINS AND THEN
C       THE BINS ARE GATHERED TOGETHER INTO NI BINS.  THE LARGEST
C       NUMBER OF VALUES IN A BIN IS FOUND, WHICH WILL DETERMINE THE
C       VERTICAL HEIGHT OF THE PRINTED HISTOGRAM.  THE HISTOGRAM IS
C       THEN PRINTED WITH THE CHARACTER 'O' DENOTING EACH OCCURRENCE.
C
C   2.  THE PARAMETER NI IS THE NUMBER OF HISTOGRAM BINS DESIRED.  THE
C       NUMBER OF COLUMNS IN THE PRINTED HISTOGRAM WILL BE 100
C       INDEPENDENT OF THE CHOICE OF NI.  IF 100 HISTOGRAM BINS ARE
C       SPECIFIED, EACH COLUMN WILL CORRESPOND TO ONE BIN.  IF NI = 25,
C       THE FIRST FOUR (100/NI) COLUMNS OF THE PRINTED HISTOGRAM WILL
C       CORRESPOND TO THE FIRST BIN.  THE VALUE FOR THE BIN WILL BE THE
C       VALUE FOUND IN EACH OF THE FIRST FOUR COLUMNS.  THE PRINTED
C       HISTOGRAM IS OUTPUT ON FORTRAN UNIT OUNIT.
C
C   INPUT PARAMETERS
C   ----------------
C
C   NPTS  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF DATA POINTS.
C
C   X     REAL VECTOR DIMENSIONED AT LEAST NPTS (UNCHANGED ON OUTPUT).
C         THE DATA VALUES.
C
C   NI    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF HISTOGRAM BINS.
C
C         NI SHOULD NOT BE GREATER THAN 100 AND SHOULD BE A FACTOR OF
C         100.  IF NI = 0 ON INPUT, THE DEFAULT VALUE OF NI = 100 WILL
C         BE USED.
C
C   P     REAL SCALAR (UNCHANGED ON OUTPUT).
C         THE DIVISOR FOR THE VERTICAL SCALE. 1/P DENOTES THE NUMBER OF
C            SYMBOLS FOR EACH OCCURRENCE IN A BIN.
C
C         IF P = 0. ON INPUT, THE DEFAULT VALUE OF P = 1. WILL BE USED.
C
C   TITLE 4-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCES
C   ----------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 47.
C
C     HARTIGAN, J. A. (1975) PRINTER GRAPHICS FOR CLUSTERING. JOURNAL OF
C        STATISTICAL COMPUTATION AND SIMULATION. VOLUME 4,PAGES 187-213.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION X(*)
      INTEGER KC, A(101), OUNIT
      CHARACTER*4 TITLE
      CHARACTER*1 B, C, H, V,AA(101)
      DATA B,C,H,V/' ','O','_','!'/
C
C     CHECK FOR DEFAULT OPTIONS
C
      IF(P.EQ.0.) P=1.
      IF(NI.EQ.0) NI=100
      IF(XMIN.GE.XMAX)  CALL RANGE(NPTS,X,XMIN,XMAX)
C
C     COMPUTE FREQUENCIES
C
      DO 10 I=1,101
   10    A(I)=0
      DO 20 J=1,NPTS
         K=KC(X(J),XMIN,XMAX,101)
   20    A(K)=A(K)+1
C
C     ADJUST FOR INTERVALS
C
      L=100/NI
      IF(L.LT.1) L=1
      DO 50 J=1,101,L
         JU=J+L-1
         JS=0
         IF(JU.GT.101) JU=101
         DO 30 K=J,JU
   30       JS=JS+A(K)
         DO 40 K=J,JU
   40       A(K)=JS/P
   50 CONTINUE
C
C     FIND MAXIMUM VALUE
C
      IAM=0
      DO 60 K=1,101
   60    IF(A(K).GT.IAM) IAM=A(K)
C
C     PRINT HISTOGRAM
C
      IF (OUNIT .LE. 0) RETURN
      MM=IAM
      WRITE(OUNIT,1) TITLE
      WRITE(OUNIT,2)
    1 FORMAT(' ',A4,' HISTOGRAM')
    2 FORMAT('+',19X,101('_'))
      DO 80 K=1,MM
         AA(1)=V
         DO 70 I=1,101
            IF(I.GT.1) AA(I)=B
            AA(101)=V
   70       IF(A(I)+K.GT.MM) AA(I)=C
         KK=MM-K+1
   80    WRITE(OUNIT,3) KK,(AA(I),I=1,101)
    3    FORMAT(10X,I5,5X,101A1)
      WRITE(OUNIT,2)
      WRITE(OUNIT,4) XMIN,XMAX
    4 FORMAT(F25.4,5X,8('!',9X),'!',F14.4)
      RETURN
      END
