      SUBROUTINE IN(MM, NN, M, N, A, CLAB, RLAB, TITLE, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      READS IN RAW DATA MATRIX AND LABELS FOR THE CASES AND VARIABLES
C      AND SETS UP THE DATA STRUCTURES NECESSARY FOR MOST OF THE
C      SUBROUTINES IN THE CLUSTER SUBLIBRARY.
C
C   DESCRIPTION
C   -----------
C
C   1.  THE DATA MUST BE ON FORTRAN UNIT 5 AND IN THE FOLLOWING FORMAT:
C
C       FIRST CARD  -- TITLE
C       SECOND CARD -- NUMBER OF VARIABLES, NUMBER OF CASES
C       THIRD CARD  -- LABELS OF VARIABLES IN A4 FORMAT SEPARATED BY ONE
C                        SPACE STARTING IN COLUMN 1
C       FOURTH CARD -- LABELS OF CASES IN A4 FORMAT SEPARATED BY ONE
C                        SPACE STARTING IN COLUMN 1
C       FIFTH CARD  -- DATA FOR EACH VARIABLE FOR THE FIRST CASE;
C                        CORRESPONDS TO THE FIRST ROW OF THE DATA MATRIX
C       SIXTH CARD  -- DATA FOR EACH VARIABLE FOR THE SECOND CASE;
C                       CORRESPONDS TO THE SECOND ROW OF THE DATA MATRIX
C       ETC..          FOR EACH SUBSEQUENT CASE
C
C     EXAMPLE
C     -------
C
C       THUG
C       4 10
C       MURD RAPE BURG AUTO
C       ATLA BOST CHIC DALL DENV DETR HART HONO HOUS KANS
C       16.50 24.80 1112. 494.0
C       4.200 13.30 982.0 954.0
C       11.60 24.70 808.0 645.0
C       18.10 34.20 1668. 602.0
C       6.900 41.50 1534. 780.0
C       13.00 35.70 1566. 788.0
C       2.500 8.800 1017. 468.0
C       3.600 12.70 1457. 637.0
C       16.80 26.60 1509. 697.0
C       10.80 43.20 1494. 765.0
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   NN    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE SECOND DIMENSION OF THE MATRIX A.  MUST BE AT LEAST N.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE OUPUT UNIT NUMBER FOR ANY ERROR MESSAGES.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   M     INTEGER SCALAR (CHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (CHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX DIMENSIONED MM BY NN (CHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (CHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (CHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (CHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 40-41.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION A(MM,NN)
      INTEGER OUNIT
      CHARACTER*4 RLAB(*), CLAB(*)
      CHARACTER*10 TITLE
C
      READ(5,2) TITLE
      READ(5,*) N, M
      IF(M.GT.MM.OR.N.GT.NN) THEN
         IF(OUNIT.GT.0) THEN
            WRITE(OUNIT,1) M, MM, N, NN
    1       FORMAT(' ERROR IN ALLOCATION ',/,
     *             I4, ' ROWS REQUESTED AND ', I4, ' ALLOCATED.',/,
     *             I4, ' COLUMNS REQUESTED AND ', I4, ' ALLOCATED.')
         ENDIF
         RETURN
      ENDIF
      READ(5,3) (CLAB(I),I=1,N)
      READ(5,3) (RLAB(I),I=1,M)
      DO 10 I=1,M
   10    READ(5,*)(A(I,J),J=1,N)
    2 FORMAT(A10)
    3 FORMAT(16(A4, 1X))
      RETURN
      END
