      SUBROUTINE KOUT(MM, M, N, A, CLAB, RLAB, TITLE, KK, DMSUM1,
     *                DMSUM2, SUM, NCLUS, DCLUS, DD, R, CWORK, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      PRINTS OUTPUT FOR K-MEANS ALGORITHM
C
C   DESCRIPTION
C   -----------
C
C   1.  THE OUTPUT CONSISTS OF THE OVERALL STATISTICS FOR THE CURRENT
C       PARTITION, FOLLOWED BY THE STATISTICS FOR EACH CLUSTER.  THE
C       ANALYSIS OF VARIANCE IS COMPUTED FOR EACH VARIABLE IN THE
C       PARTITION.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   KK    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CLUSTERS.
C
C   DMSUM1 INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX SUM.  MUST BE AT LEAST 7.
C
C   DMSUM2 INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE SECOND DIMENSION OF THE MATRIX SUM.  MUST BE AT LEAST N.
C
C   SUM   REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMSUM1, WHOSE SECOND
C            DIMENSION MUST BE DMSUM2, AND WHOSE THIRD DIMENSION MUST
C            BE AT LEAST K+1 (UNCHANGED ON OUTPUT).
C         THE PARAMETERS FOR EACH CLUSTER.
C
C         SUM(1,J,I) IS THE VALUE OF THE J-TH VARIABLE AT THE CENTER OF
C                       CLUSTER I
C         SUM(2,J,I) IS THE NUMBER OF NON-MISSING OBSERVATIONS FOR THE
C                       J-TH VARIABLE IN CLUSTER I
C         SUM(3,J,I) IS THE MEAN OF THE J-TH VARIABLE IN CLUSTER I
C         SUM(4,J,I) IS THE STANDARD DEVIATION OF THE J-TH VARIABLE IN
C                       CLUSTER I
C         SUM(5,J,I) IS THE MINIMUM OF THE J-TH VARIABLE IN CLUSTER I
C         SUM(6,J,I) IS THE MAXIMUM OF THE J-TH VARIABLE IN CLUSTER I
C         SUM(7,J,I) IS THE SUM OF SQUARED DEVIATIONS FOR THE J-TH
C                       VARIABLE FROM THE MEAN OF CLUSTER I
C
C         THE K+1-ST ROW OF SUM STORES THE SAME CALCULATIONS AS ABOVE
C            EXCEPT FOR THE ENTIRE DATA SET RATHER THAN FOR AN
C            INDIVIDUAL CLUSTER
C
C   NCLUS INTEGER VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         NCLUS(I) IS THE CLUSTER FOR CASE I.
C
C   DCLUS REAL VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         DCLUS(I) IS THE DISTANCE OF EACH CASE TO THE CLOSEST CLUSTER.
C
C   DD    REAL VECTOR DIMENSIONED AT LEAST N.
C         WORK VECTOR.
C
C   R     REAL VECTOR DIMENSIONED AT LEAST N.
C         WORK VECTOR.
C
C   CWORK VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C         WORK VECTOR.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 110.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMSUM1, DMSUM2, OUNIT
      DIMENSION SUM(DMSUM1,DMSUM2,*), NCLUS(*), DCLUS(*), A(MM,*), R(*),
     *          DD(*)
      CHARACTER*4 CLAB(*), RLAB(*), CWORK(*)
      CHARACTER*10 TITLE
      DATA NPAGE,LC/0,0/
C
C     OUTPUT MEAN SQUARE CALCULATION OVER ALL CLUSTERS
C
      NPAGE=NPAGE+1
      WRITE(OUNIT,1) NPAGE
    1 FORMAT('1',110X,I5)
      WRITE(OUNIT,2) KK
    2 FORMAT(' OVERALL MEAN SQUARE CALCULATIONS, FOR EACH VARIABLE, ',
     *' WITH',I5,'  CLUSTERS')
      ASSW=0.
      DO 20 J=1,N
         SD=0.
         SC=0.
         SSB=0.
         SSW=0.
         DO 10 K=1,KK
            SD=SD+SUM(3,J,K)*SUM(2,J,K)
            SSB=SSB+SUM(3,J,K)**2*SUM(2,J,K)
            SSW=SSW+SUM(7,J,K)
   10       SC=SC+SUM(2,J,K)
         DFB=KK-1.
         DFW=SC-DFB-1.
         ASSW=ASSW+SSW
         IF(SC.GT.0.) SSB=SSB-SD**2/SC
         IF(DFB.GT.0.) SSB=SSB/DFB
         IF(DFW.GT.0.) SSW=SSW/DFW
         RATIO=0.
         IF(LC.NE.0.AND.SSW.GT.0.) RATIO=(R(J)/SSW-1.)*(1.+DFW)+1.
         R(J)=SSW
         WRITE(OUNIT,3)CLAB(J),SSW,DFW,SSB,DFB,RATIO
    3    FORMAT(' VARIABLE',4X,A4,F20.6,'(WITHIN MEAN SQ.)',F4.0,'(WITHI
     *N DF)',F20.6,'(BETWEEN MSQ)',F4.0,'(BETWEEN DF)',F6.1,'(FRATIO)')
   20 CONTINUE
      WRITE(OUNIT,4) ASSW
    4 FORMAT(' OVERALL WITHIN SUM OF SQUARES',F20.6)
      LC=LC+1
C
C     OUTPUT STATISTICS FOR EACH CLUSTER
C
      DO 50 K=1,KK
         WRITE(OUNIT,5)
    5    FORMAT(1X,131('-'))
         WRITE(OUNIT,6) K,KK
    6    FORMAT(I5,'   TH CLUSTER OF',I5)
         WRITE(OUNIT,7)(I,I=1,10)
    7    FORMAT('0CLUSTER MEMBERS WITH THEIR DISTANCES TO THE CLUSTER CE
     *NTRE'/13X,10I11)
         L=0
         DO 30 I=1,M
            IF(NCLUS(I).EQ.K) THEN
               L=L+1
               CWORK(L)=RLAB(I)
               DD(L)=DCLUS(I)
            ENDIF
            IF ((L.GE.10.OR.I.GE.M).AND.L.NE.0) THEN
               WRITE(OUNIT,8)(CWORK(LL),LL=1,L)
               WRITE(OUNIT,9)(DD(LL),LL=1,L)
    8          FORMAT(15X,10(7X,A4))
    9          FORMAT(15X,10F11.4)
               L=0
            ENDIF
   30    CONTINUE
         WRITE(OUNIT,11)
   11    FORMAT('0SUMMARY STATISTICS FOR THE CLUSTER')
         WRITE(OUNIT,12)
   12    FORMAT(' LABEL',5X,'CENTRE',8X,'COUNT',12X,'AVE',
     *           13X,'SD',11X,'XMIN',11X,'XMAX',12X,'SSQ')
         DO 40 J=1,N
   40       WRITE(OUNIT,13)CLAB(J),(SUM(I,J,K),I=1,7)
   13       FORMAT(1X,A4,7F15.6)
   50 CONTINUE
      RETURN
      END
