      SUBROUTINE LINE(MM, M, N, A, CLAB, RLAB, TITLE, R, S, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      PRODUCES A LINE PROFILE OF THE VARIABLES FOR EACH CASE
C
C   DESCRIPTION
C   -----------
C
C   1.  THE LINE PROFILE IS A HORIZONTAL PROFILE THAT PRESENTS A SCALED
C       PLOT OF THE VARIABLES FOR EACH CASE.  THE VALUES ARE SCALED
C       SUCH THAT THE SMALLEST VALUE FOR A VARIABLE OVER ALL CASES IS
C       PLOTTED AT THE EXTREME LEFT AND THE LARGEST VALUE AT THE
C       EXTREME RIGHT.  IF ANY TWO VALUES OCCUPY THE SAME LOCATION ON
C       THE LINE, ONE OF THE VALUES IS OFFSET ONE LOCATION.  OUTPUT IS
C       ON FORTRAN UNIT OUNIT.  THERE MUST BE NO MORE THAN 35
C       VARIABLES.
C
C   2.  TO OBTAIN A PROFILE OF THE CASES FOR EACH VARIABLE, SEE CLUSTER
C       SUBROUTINE PROF.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.  N MUST BE LESS THAN 36.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND SECOND
C            DIMENSION MUST BE AT LEAST N  (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   R     REAL MATRIX WHOSE FIRST DIMENSION MUST BE 2 AND SECOND
C            DIMENSION IS AT LEAST N  (UNCHANGED ON OUTPUT).
C         R(1,J) DENOTES THE SMALLEST VALUE OF THE J-TH VARIABLE OVER
C                ALL OF THE CASES.
C         R(2,J) DENOTES THE LARGEST VALUE OF THE J-TH VARIABLE OVER
C                ALL OF THE CASES.
C
C         IF R(1,1) = 0. ON INPUT, THE SUBROUTINE WILL CALCULATE THE
C            MATRIX.  OTHERWISE, THE ROUTINE WILL USE THE VALUES
C            SUPPLIED ON INPUT.
C
C   S     VECTOR OF 1-CHARACTER VARIABLES DIMENSIONED AT LEAST N
C            (UNCHANGED ON OUTPUT).
C         S(I) IS THE ONE-CHARACTER SYMBOL USED TO DENOTE VARIABLE I.
C
C         IF S(1) IS '0' ON INPUT, THE ROUTINE WILL LABEL THE FIRST
C            VARIABLE '1', THE SECOND VARIABLE '2', ETC.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 39, 44.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION A(MM,*), R(2,*)
      INTEGER OUNIT
      CHARACTER*4 CLAB(*), RLAB(*)
      CHARACTER*10 TITLE
      CHARACTER*1 SA(36), AA(105), S(*), P1, P2
      DATA P1,P2/'(',')'/
      DATA SA/'1','2','3','4','5','6','7','8','9',
     *        'A','B','C','D','E','F','G','H','I',
     *        'J','K','L','M','N','O','P','Q','R',
     *        'S','T','U','V','W','X','Y','Z','.'/
C
C     COMPUTE RANGES AND SYMBOLS IF REQUESTED
C
      IF(R(1,1).EQ.0.) THEN
         DO 20 J=1,N
            R(1,J)=A(1,J)
            R(2,J)=A(1,J)
            DO 10 I=1,M
               IF(A(I,J).LT.R(1,J)) R(1,J)=A(I,J)
   10          IF(A(I,J).GT.R(2,J)) R(2,J)=A(I,J)
            IF(R(1,J).EQ.R(2,J)) R(1,J)=R(1,J)-1.
   20    CONTINUE
      ENDIF
C
      IF(S(1).EQ.'0') THEN
         DO 30 J=1,N
            JJ=J-((J-1)/35)*35
   30       S(J)=SA(JJ)
      ENDIF
C
      IF (OUNIT .LE. 0) RETURN
      WRITE(OUNIT,1) TITLE
      WRITE(OUNIT,2)
      WRITE(OUNIT,3)(CLAB(J),S(J),R(1,J),R(2,J),J=1,N)
      WRITE(OUNIT,4)
    1 FORMAT('1 LINE PRINT OF ARRAY ',A10)
    2 FORMAT('0 VARIABLE  SYMBOL       MIN       MAX ')
    3 FORMAT(3X,A4,5X,A1,5X,2F10.4)
    4 FORMAT('0')
C
C     PRINT CASES
C
      DO 70 I=1,M
         DO 40 J=1,105
   40       AA(J)=SA(36)
         DO 60 J=1,N
            JJ=100*(A(I,J)-R(1,J))/(R(2,J)-R(1,J))+3
   50       IF(JJ.LT.3) JJ=1
            IF(JJ.GT.103) JJ=105
            IF(AA(JJ).NE.SA(36)) THEN
               IF(JJ.LT.51) JJ=JJ+1
               IF(JJ.GT.51) JJ=JJ-1
               IF(JJ.NE.51) GO TO 50
            ENDIF
   60       AA(JJ)=S(J)
         AA(2)=P1
         AA(104)=P2
         WRITE(OUNIT,5) RLAB(I),(AA(J),J=1,105)
    5    FORMAT(1X,A4,5X,105A1)
   70 CONTINUE
      RETURN
      END
