      SUBROUTINE MISS(MM, M, N, A, NC)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      REPLACES A MISSING DATA VALUE BY THE CLUSTER MEAN
C
C   DESCRIPTION
C   -----------
C
C   1.  THE MEANS FOR THE NON-MISSING VALUES WITHIN EACH CLUSTER ARE
C       DETERMINED AND ANY MISSING VALUES ARE REPLACED BY THE
C       APPROPRIATE CLUSTER MEAN.  MISSING VALUES ARE DENOTED BY 99999.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N. (CHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   NC    INTEGER VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         NC(I) INDICATES THE CLUSTER NUMBER FOR CASE I.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 69, 72.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION A(MM,*),NC(*)
C
C     FIND NUMBER OF CLUSTERS
C
      XM=99999.
      K=1
      DO 10 I=1,M
   10    IF(NC(I).GT.K) K=NC(I)
      DO 40 L=1,K
         DO 40 J=1,N
C
C     COMPUTE MEAN FOR CLUSTER
C
            PR=0.
            AV=0.
            DO 20 I=1,M
               IF(NC(I).EQ.L.AND.A(I,J).NE.XM) THEN
                  PR=PR+1.
                  AV=AV+A(I,J)
               ENDIF
   20       CONTINUE
            IF(PR.NE.0.) AV=AV/PR
C
C      REPLACE MISSING VALUES
C
            DO 30 I=1,M
   30          IF(NC(I).EQ.L.AND.A(I,J).EQ.XM) A(I,J)=AV
   40 CONTINUE
      RETURN
      END
