       SUBROUTINE MOM(MM, M, N, A, ICLUS, W, U, DMC1, DMC2, C)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      COMPUTES WEIGHTED MEANS AND COVARIANCES
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C   ICLUS INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE CLUSTER NUMBER.
C
C   W     REAL VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         VECTOR OF WEIGHTS FOR THE OBJECTS.
C
C   DMC1  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX C.  MUST BE AT LEAST N.
C
C   DMC2  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE SECOND DIMENSION OF THE MATRIX C.  MUST BE AT LEAST N.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   U     REAL VECTOR DIMENSIONED AT LEAST N.
C         VECTOR OF WEIGHTED CLUSTER MEANS FOR EACH VARIABLE.
C
C   C     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMC1, WHOSE SECOND
C            DIMENSION MUST BE DMC2, AND WHOSE THIRD DIMENSION MUST BE
C            AT LEAST K.
C         C(I,J,K) IS THE IJ-TH ELEMENT OF THE COVARIANCE MATRIX FOR THE
C            K-TH CLUSTER.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 73.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMC1, DMC2
      DIMENSION C(DMC1,DMC2,*), W(*), U(*), A(MM,*)
C
      SP=0.
      DO 10 I=1,M
   10    SP=SP+W(I)
      IF(SP.EQ.0.) SP=R1MACH(4)
C
C     COMPUTED WEIGHTED MEANS
C
      DO 30 J=1,N
         SS=0.
         DO 20 I=1,M
   20       SS=SS+A(I,J)*W(I)
   30    U(J)=SS/SP
C
C     COMPUTED WEIGHTED COVARIANCES
C
      DO 50 J=1,N
         DO 50 K=1,J
            SS=0.
            DO 40 I=1,M
   40          SS=SS+(A(I,J)-U(J))*(A(I,K)-U(K))*W(I)
            C(J,K,ICLUS)=SS/SP
   50       C(K,J,ICLUS)=C(J,K,ICLUS)
      RETURN
      END
