      SUBROUTINE MTREE(M, K, MT, MS, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      OUTPUTS A TREE OF OBJECTS
C
C   DESCRIPTION
C   -----------
C
C   1.  THE TREE IS STORED IN THE ARRAY MT SUCH THAT MT(I) IS THE
C       ANCESTOR NODE OF OBJECT I FOR I=1,...,M AND MT(I) IS THE
C       ANCESTOR NODE FOR NODE I FOR I=M+1,...,K.
C
C   2.  TO INTERPRET THE OUTPUT, EACH COLUMN CORRESPONDS TO THE OBJECT
C       WHOSE NUMBER IS THE FIRST ENTRY IN THE COLUMN.  THE SECOND
C       NUMBER IN EACH COLUMN IS THE ANCESTOR NODE OF THE OBJECT, THE
C       THIRD NUMBER IS THE ANCESTOR NODE OF THE SECOND NUMBER, ETC.
C       THE FINAL TREE CAN BE RECOVERED BY CONNECTING EACH NODE TO ITS
C       ANCESTOR AND LINKING THE ANCESTORS TO THEIR ANCESTORS UNTIL
C       ONLY ONE NODE IS LEFT.
C
C   INPUT PARAMETERS
C   ----------------
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF OBJECTS.
C
C   K     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF NODES IN TREE.
C
C   MT    INTEGER VECTOR DIMENSIONED AT LEAST K (UNCHANGED ON OUTPUT).
C         MT(I) IS THE ANCESTOR OF OBJECT I IN THE TREE.
C
C   MS    INTEGER VECTOR DIMENSIONED AT LEAST M.
C         WORK VECTOR.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 189.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER OUNIT
      DIMENSION MT(*), MS(*)
C
      IF (OUNIT .LE. 0) RETURN
      NC=1
      DO 10 I=1,M
   10    MS(I)=I
      WRITE(OUNIT,1)(MS(I),I=1,M)
    1 FORMAT(1X,40I3)
   20 IF(NC.NE.0) THEN
         NC=0
         DO 30 I=1,M
            J=MS(I)
            MS(I)=MT(J)
            IF(MS(I).LE.J.AND.J.NE.K) RETURN
   30       IF(MS(I).NE.K) NC=1
         WRITE(OUNIT,1)(MS(I),I=1,M)
         GO TO 20
      ENDIF
      RETURN
      END
