      SUBROUTINE OUT1(MM, M, N, A, CLAB, RLAB, TITLE, KC, DMNB, NB,
     *                OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      PRINTS BLOCK DIAGRAM FOUND ON TOP OF PAGE 51 OF USER GUIDE
C
C   DESCRIPTION
C   -----------
C
C   1.  EACH BLOCK IS ASSUMED TO CONTAIN ONLY ONE VARIABLE.  NOT
C       PRESENTLY USED BY ANY CLUSTER ROUTINES.  OUTPUT DIAGRAM IS VERY
C       SIMILAR TO DIAGRAM FROM CLUSTER ROUTINE BLOCK.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF MATRIX A.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF OBJECTS.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND SECOND
C            DIMENSION MUST BE AT LEAST M (UNCHANGED ON OUTPUT).
C         THE DATA MATRIX.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N
C            (UNCHANGED ON OUTPUT).
C         ORDERED LABELS OF THE COLUMNS.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M
C            (UNCHANGED ON OUTPUT).
C         ORDERED LABELS OF THE ROWS.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   KC    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF BLOCKS.
C
C   DMNB  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF MATRIX NB.  MUST BE AT LEAST 4.
C
C   NB    REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMNB AND SECOND
C            DIMENSION MUST BE AT LEAST KC (UNCHANGED ON OUTPUT).
C         THE MATRIX DEFINING THE BOUNDARIES OF THE BLOCKS.
C
C         NB(1,I) IS 1 + THE FIRST ROW IN BLOCK I
C         NB(2,I) IS 1 + THE LAST ROW IN BLOCK I
C         NB(3,I) IS 1 + THE FIRST COLUMN IN BLOCK I
C         NB(4,I) IS 1 + THE LAST COLUMN IN BLOCK I
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR ERROR MESSAGES.
C
C   REFERENCES
C   ----------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 273.
C
C     HARTIGAN, J. A. (1975) PRINTER GRAPHICS FOR CLUSTERING. JOURNAL OF
C        STATISTICAL COMPUTATION AND SIMULATION. VOLUME 4,PAGES 187-213.
C
C     DALLAL, GERARD E.  A USER'S GUIDE TO J. A. HARTIGAN'S "CLUSTERING
C        ALGORITHMS".  YALE UNIVERSITY.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMNB, OUNIT
      DIMENSION A(MM,*), NB(DMNB,*)
      CHARACTER*10 TITLE
      CHARACTER*4 CLAB(*), RLAB(*), AA(20), BLANK, DASH
      DATA BLANK,DASH/'    ','----'/
C
      IF (OUNIT .LE. 0) RETURN
      WRITE(OUNIT,1)TITLE
    1 FORMAT('1 ONE WAY SPLIT OF ',A10)
      NN=(N-1)/20+1
      DO 40 LL=1,NN
         JL=(LL-1)*20+1
         JU=LL*20
         IF(JU.GT.N) JU=N
         WRITE(OUNIT,2) TITLE, (CLAB(J),J=JL,JU)
         WRITE(OUNIT,3)(DASH,J=JL,JU+1)
    2    FORMAT(2X,A10,20(A4,2X))
    3    FORMAT(2X,A4,6X,20(A4,2X))
         DO 30 I=1,M
            WRITE(OUNIT,4)RLAB(I),(A(I,J),J=JL,JU)
    4       FORMAT(1X,A4,5X,20F6.2/(7X,20F6.2))
            DO 10 L=1,20
   10          AA(L)=BLANK
            NC=0
            DO 20 K=1,KC
               L=0
               IF(NB(2,K).EQ.I+1) L=NB(3,K)-1
               IF(L.GE.JL.AND.L.LE.JU+1) AA(L-JL+2)=DASH
   20          IF(L.NE.0) NC=1
            IF(NC.NE.0) WRITE(OUNIT,3)(AA(L),L=1,20)
   30    CONTINUE
         IF (LL .NE. 1) WRITE(OUNIT,5)
    5    FORMAT('1')
   40 CONTINUE
      RETURN
      END
