      SUBROUTINE PMUT(DMB, M, N, B, BCLAB, BRLAB, NR, NC, DMA, A, CLAB,
     *                RLAB, CWORK)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      PERMUTES AN ARRAY ACCORDING TO PERMUTATION VECTORS
C
C   INPUT PARAMETERS
C   ----------------
C
C   DMB   INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF MATRIX B.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   B     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMB AND SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE ORIGINAL DATA.
C
C   BCLAB INTEGER VECTOR DIMENSIONED AT LEAST N (UNCHANGED ON OUTPUT).
C         UNORDERED LABELS OF THE COLUMNS.
C
C   BRLAB INTEGER VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         UNORDERED LABELS OF THE ROWS.
C
C   NR    INTEGER VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         PERMUTATION VECTOR FOR THE ROWS.
C
C   NC    INTEGER VECTOR DIMENSIONED AT LEAST N (UNCHANGED ON OUTPUT).
C         PERMUTATION VECTOR FOR THE COLUMNS.
C
C   DMA   INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF MATRIX A.  MUST BE AT LEAST M.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMA AND SECOND
C            DIMENSION MUST BE AT LEAST N (CHANGED ON OUTPUT).
C         THE PERMUTED DATA.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N
C            (UNCHANGED ON OUTPUT).
C         ORDERED LABELS OF THE COLUMNS.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M
C            (UNCHANGED ON OUTPUT).
C         ORDERED LABELS OF THE ROWS.
C
C   CWORK VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST
C            MAX(M,N).
C         WORK VECTOR.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 298.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMA, DMB
      DIMENSION A(DMA,*), B(DMB,*), NR(*), NC(*)
      CHARACTER*4 CLAB(*), RLAB(*), CWORK(*)
      INTEGER BCLAB(*), BRLAB(*)
C
      DO 10 I = 1 , M
   10    CWORK(I) = RLAB(I)
      DO 20 I=1,M
         K=NR(I)
         RLAB(I) = CWORK(K)
         DO 20 J=1,N
   20       A(I,J)=B(K,J)
      DO 30 J=1,N
         DO 30 I=1,M
   30       B(I,J)=A(I,J)
      DO 40 I = 1 , N
   40    CWORK(I) = CLAB(I)
      DO 50 J=1,N
         K=NC(J)
         CLAB(J) = CWORK(K)
         DO 50 I=1,M
   50       A(I,J)=B(I,K)
      RETURN
      END
