      SUBROUTINE RANK(MM, M, N, A, CLAB, RLAB, TITLE, IWORK, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      RANKS CASES FOR EACH VARIABLE
C
C   DESCRIPTION
C   -----------
C
C   1.  THE VALUES OF EACH VARIABLE ARE TRANSFORMED INTO INTEGER RANKS
C       AND THE CASES ARE PRINTED OUT BY INCREASING RANK.  THIS IS
C       ACCOMPLISHED BY SORTING WITH RESPECT TO EACH VARIABLE AND
C       PRINTING OUT THE CASES CORRESPONDING TO THE FINAL SORTED ORDER
C       ON FORTRAN UNIT OUNIT.
C
C   2.  THE SORTING IS BASICALLY A BUBBLE SORT AND REQUIRES M*(M-1)
C       COMPARISONS FOR EACH VARIABLE.  THEREFORE, THE SUBROUTINE
C       SHOULD NOT BE USED FOR M > 50.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND SECOND
C            DIMENSION MUST BE AT LEAST N  (DESTROYED DURING EXECUTION).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   IWORK INTEGER VECTOR DIMENSIONED AT LEAST M.
C         WORK VECTOR.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 32-39, 43.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION A(MM,*), IWORK(*)
      INTEGER OUNIT
      CHARACTER*4 CLAB(*), RLAB(*)
      CHARACTER*10 TITLE
C
      IF (OUNIT .GT. 0) THEN
         WRITE(OUNIT,1) TITLE
         WRITE(OUNIT,2)(CLAB(J),J=1,N)
         WRITE(OUNIT,3)
      ENDIF
      DO 40 J=1,N
C
C     ORDER VALUES OF THE JTH COLUMN
C
         DO 10 I = 1 , M
   10       IWORK(I) = I
         DO 20 I=1,M
            DO 20 II=1,I-1
               IF(A(IWORK(I),J).LT.A(IWORK(II),J)) THEN
                  JJ=IWORK(I)
                  IWORK(I)=IWORK(II)
                  IWORK(II)=JJ
               ENDIF
   20    CONTINUE
C
C     STORE J-TH COLUMN OF POINTERS
C
         DO 30 I = 1 , M
   30       A(I,J) = REAL(IWORK(I))
C
   40 CONTINUE
C
C     OUTPUT RANKS
C
      IF (OUNIT .GT. 0) THEN
         DO 50 I = 1 , M
   50       WRITE(OUNIT,2) (RLAB(INT(A(I,J))),J=1,N)
    1    FORMAT('1 RANKING OF ARRAY ', A10)
    2    FORMAT(18(1X,A4,2X))
    3    FORMAT('0')
      ENDIF
      RETURN
      END
