      SUBROUTINE RELOC(MM, M, N, A, K, NC, DMU, U, WORK)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      SETS EACH CLUSTER CENTER EQUAL TO THE CLUSTER MEAN.  CAN BE USED
C      WITH CLUSTER SUBROUTINE ASSIGN TO FORM A USER-DEFINED K-MEANS
C      PACKAGE
C
C   DESCRIPTION
C   -----------
C
C   1.  THE VARIABLES MUST BE SCALED SIMILARLY (CLUSTER SUBROUTINE
C       STAND CAN BE USED TO STANDARDIZE THE VARIABLES).
C
C   2.  MISSING VALUES ARE DENOTED BY 99999.  AND DO NOT AFFECT THE
C       COMPUTATION OF THE VARIABLE MEANS.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   K     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CLUSTERS.
C
C   NC    INTEGER VECTOR DIMENSIONED AT LEAST M (UNCHANGED ON OUTPUT).
C         NC(I) IS THE CLUSTER FOR CASE I.
C
C   DMU   INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX U.  MUST BE AT LEAST N.
C
C   WORK  REAL VECTOR DIMENSIONED AT LEAST N.
C         WORK VECTOR.
C
C   OUTPUT PARAMETER
C   ----------------
C
C   U     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMU AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST K.
C         U(I,J) STORES THE VALUE OF VARIABLE I FOR THE CENTER OF
C            CLUSTER J, (IE. THE MEAN VALUE OF VARIABLE I OVER THE
C            CASES IN CLUSTER J).
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 84-105.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMU
      DIMENSION A(MM,*), U(DMU,*), NC(*), WORK(*)
      XM=99999.
C
C     COMPUTE MEANS
C
      DO 50 L=1,K
         DO 10 J=1,N
            WORK(J)=0.
   10       U(J,L)=0.
         DO 30 I = 1 , M
            IF(NC(I).EQ.L) THEN
               DO 20 J=1,N
                  IF (A(I,J) .NE. XM) THEN
                     WORK(J)=WORK(J)+1.
                     U(J,L)=U(J,L)+A(I,J)
                  ENDIF
   20          CONTINUE
            ENDIF
   30    CONTINUE
         DO 40 J = 1 , N
            IF(WORK(J).NE.0.) U(J,L)=U(J,L)/WORK(J)
            IF(WORK(J).EQ.0.) U(J,L)=XM
   40    CONTINUE
   50 CONTINUE
      RETURN
      END
