      SUBROUTINE SCALE(MM, M, N, A, KL, KK)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      DISCRETIZES THE DATA INTO CLASSES
C
C   DESCRIPTION
C   -----------
C
C   1.  THE MINIMUM AND MAXIMUM VALUES ARE FOUND AND EACH VALUE IS THEN
C       PLACED INTO ONE OF KL INTERVALS OF EQUAL LENGTH BETWEEN THE
C       MINIMUM AND MAXIMUM VALUES.
C
C   2.  THE USER HAS THE OPTION TO EITHER SCALE UNIFORMLY OVER ALL DATA
C       VALUES, OR TO UNIFORMLY SCALE EACH VARIABLE.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (CHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES WHICH WILL BE SCALED ON OUTPUT.
C
C   KL    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         MAXIMUM SCALED VALUE.
C
C   KK    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         SCALING OPTION.
C
C         IF KK = 1   DATA ARE SCALED UNIFORMLY OVER ALL VALUES
C         IF KK = 2   DATA ARE SCALED UNIFORMLY OVER EACH VARIABLE
C
C   OUTPUT PARAMETER
C   ----------------
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N.
C         THE MATRIX OF SCALED DATA VALUES.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 148, 151.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION A(MM,*)
C
      IF(KK.EQ.2) THEN
C
C     SCALE OVER EACH VARIABLE
C
         DO 30 J=1,N
C
C     COMPUTE MINIMUM AND MAXIMUM
C
            XMIN=A(1,J)
            XMAX=A(1,J)
            DO 10 I=1,M
               IF(A(I,J).GT.XMAX) XMAX=A(I,J)
               IF(A(I,J).LT.XMIN) XMIN=A(I,J)
   10       CONTINUE
            IF(XMIN.EQ.XMAX) XMAX=XMIN+.000001
C
C     SCALE VALUES
C
            ZZ=KL/(XMAX-XMIN)
            DO 20 I=1,M
               XK=(A(I,J)-XMIN)*ZZ+1
               IF (A(I,J) .EQ. XMAX) XK = KL+.9999
               A(I,J)=XK
   20       CONTINUE
   30    CONTINUE
C
      ELSE
C
C     SCALE OVER ALL VALUES
C
C     FIND MINIMUM AND MAXIMUM
C
         XMIN=A(1,1)
         XMAX=A(1,1)
         DO 40 I=1,M
            DO 40 J=1,N
               IF(A(I,J).LT.XMIN) XMIN=A(I,J)
   40    IF(A(I,J).GT.XMAX) XMAX=A(I,J)
         IF(XMIN.EQ.XMAX) XMAX=XMIN+.000001
C
C     SCALE VALUES
C
         ZZ=KL/(XMAX-XMIN)
         DO 50 I=1,M
            DO 50 J=1,N
               XK=(A(I,J)-XMIN)*ZZ+1.
               IF (A(I,J) .EQ. XMAX) XK = KL+.99999
               A(I,J)=XK
   50    CONTINUE
      ENDIF
      RETURN
      END
