      SUBROUTINE SPARSE(MM, M, N, A, CLAB, RLAB, TITLE, KK, DMCOV, COV,
     *                  DMWRK1, WORK1, DMWRK2, WORK2, CWORK, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C     APPROXIMATES THE COVARIANCE MATRIX BY BB' WHERE B CONTAINS MANY
C     ZEROS AND CORRESPONDS TO THE LOADING MATRIX FOR FACTOR ANALYSIS
C     WHERE CLUSTERS OF VARIABLES ARE THE FACTORS
C
C   DESCRIPTION
C   -----------
C
C   1.  THE FIRST COLUMN OF B IS THE EIGENVECTOR CORRESPONDING TO THE
C       LARGEST EIGENVALUE OF THE COVARIANCE MATRIX.  THEN THE ROW THAT
C       MINIMIZES THE SQUARE OF THE CORRELATION OF THAT VARIABLE WITH
C       THE WEIGHTED AVERAGE OF THE ORIGINAL VARIABLES, WEIGHTED BY THE
C       FIRST COLUMN OF B IS FOUND.  THAT ROW IS REMOVED FROM THE
C       COVARIANCE MATRIX WITH ALL CORRELATIONS BEING REPLACED BY THE
C       PARTIAL CORRELATION WITH THE REMOVED ROW FIXED AND BY SETTING
C       ALL CORRELATIONS INVOLVING THAT ROW TO ZERO.  THE SECOND COLUMN
C       OF B IS THE EIGENVECTOR FOR THE NEW MATRIX AND THE PROCESS
C       CONTINUES.
C
C   2.  THE ORIGINAL COVARIANCE MATRIX IS OUTPUTED, FOLLOWED BY THE
C       MATRIX B, THE LOADING MATRIX FOR THE FACTOR ANALYSIS.  EACH
C       COLUMN OF THE LOADING MATRIX CORRESPONDS TO A CLUSTER OF
C       VARIABLES WITH THE ENTRIES CORRESPONDING TO THE FACTORS.  ONLY
C       VARIABLES WITH NON-ZERO FACTORS ARE ELEMENTS OF THE CLUSTER.
C       THEN THE RESIDUAL MATRIX (A - BB') IS PRINTED, THEN THE
C       PERCENTAGE OF VARIANCE FOR EACH CLUSTER, BOTH ACTUAL AND
C       CUMULATIVE, ARE PRINTED OUT, ALL ON FORTRAN UNIT OUNIT.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N. (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   KK    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF DESIRED FACTORS.  SHOULD BE LESS THAN N.
C
C   DMCOV INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF THE MATRIX COV.  MUST BE AT LEAST N.
C
C   COV   REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMCOV AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST N.
C         WORK MATRIX.  USED TO STORE COVARIANCE MATRIX AT EACH
C            ITERATION.
C
C   DMWRK1 INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF THE MATRIX WORK1.  MUST BE AT LEAST
C            N.
C
C   WORK1 REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMWRK1 AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST N+1.
C         WORK MATRIX.
C
C   DMWRK2 INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF THE MATRIX WORK2.  MUST BE AT LEAST
C            N.
C
C   WORK2 REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMWRK2 AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST N+2.
C         WORK MATRIX.
C
C   CWORK VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST 2*N.
C         WORK VECTOR.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 313-329.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMWRK1, DMWRK2, DMCOV, OUNIT
      DIMENSION COV(DMCOV,*), WORK1(DMWRK1,*), WORK2(DMWRK2,*), A(MM,*)
      CHARACTER*4 BLANK, CWORK(*), CLAB(*),RLAB(*)
      CHARACTER*10 TITLE, CTITLE, RTITLE, BTITLE, AVETIT
      DATA BLANK / '    ' /
C
C     COMPUTE COVARIANCE MATRIX
C
      CALL TWO(MM,M,N,A,CLAB,RLAB,TITLE,DMCOV,COV,CWORK(1),CTITLE,
     1         WORK1(1,1),CWORK(N+1),CWORK(N+1),AVETIT)
C
C     OUTPUT COVARIANCE MATRIX
C
      IF (OUNIT .GT. 0) WRITE(OUNIT,1)
    1 FORMAT('1COVARIANCE MATRIX ')
      CALL OUT(DMCOV,N,N,COV,CWORK(1),CWORK(1),CTITLE,OUNIT)
      SS=0.
      DO 10 J=1,N
   10    SS=SS+COV(J,J)
      DO 40 KR=1,KK
         CWORK(N+KR)=BLANK
         DO 20 I=1,N
            DO 20 J=1,N
   20           WORK2(I,J)=COV(I,J)
         CALL FIND(DMCOV,N,COV,WORK1(1,KR),WORK2(1,N+1),WORK2(1,N+2))
C
C     REPLACE COVARIANCE MATRIX BY RESIDUALS
C
         DO 30 I=1,N
            DO 30 J=1,N
   30          COV(I,J)=WORK2(I,J)-WORK1(I,KR)*WORK1(J,KR)
   40 CONTINUE
C
C     OUTPUT RESIDUAL MATRIX
C
      IF (OUNIT .GT. 0) WRITE(OUNIT,2)
    2 FORMAT('0RESIDUAL MATRIX ')
      RTITLE = ' RESIDUAL '
      CALL OUT(DMWRK2,N,N,WORK2,CWORK(1),CWORK(1),RTITLE,OUNIT)
C
C     OUTPUT LOADING MATRIX
C
      IF (OUNIT .GT. 0) WRITE(OUNIT,3)
    3 FORMAT('0SPARSE ROOT OF COVARIANCE MATRIX ')
      BTITLE = '  SPARSE  '
      CALL OUT(DMWRK1,N,KK,WORK1,CWORK(N+1),CWORK(1),BTITLE,OUNIT)
C
C     COMPUTE AND OUTPUT VARIANCES OF FACTORS
C
      DO 50 K=1,KK
         WORK1(N+1,K)=0.
         DO 50 J=1,N
   50       WORK1(N+1,K)=WORK1(N+1,K)+WORK1(J,K)**2
      IF (OUNIT .GT. 0) WRITE(OUNIT,4) SS,(WORK1(N+1,K),K=1,KK)
    4 FORMAT(' VARIANCES, TOTAL AND FACTORS '/
     *F10.4,/,4X,10F11.4,/(4X,10F11.4))
      DO 60 K=2,KK
   60    WORK1(N+1,K)=WORK1(N+1,K)+WORK1(N+1,K-1)
      DO 70 K=1,KK
   70    WORK1(N+1,K)=100.*WORK1(N+1,K)/SS
      IF (OUNIT .GT. 0) WRITE(OUNIT,5)(WORK1(N+1,K),K=1,KK)
    5 FORMAT(' PERCENTAGE CUMULATIVE VARIANCE '/(4X,10F11.4))
      RETURN
      END
