      SUBROUTINE TRIAD(IOBJ, M, K, MT, DMT1, DMT2, T, DMULT, ULT, TT,
     *                 FT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      COMPUTES FALSE AND TRUE TRIADS INVOLVING IOBJ-TH OBJECT
C
C   DESCRIPTION
C   -----------
C
C   1.  IF IOBJ .LE.  M, THE NUMBER OF FALSE AND TRUE TRIADS FOR THE
C       IOBJ-TH OBJECT ARE COMPUTED; OTHERWISE, THE NUMBER OF FALSE AND
C       TRUE TRIADS FOR ALL OBJECTS ARE COMPUTED.
C
C   2.  FOR EACH OBJECT, THE ULTRAMETRIC IS COMPUTED.  THE ULTRAMETRIC
C       BETWEEN NODES I AND J, STORED IN ULT(I,J), IS THE FIRST
C       ANCESTOR IN COMMON TO BOTH NODES.  THEN EVERY PAIR OF
C       ULTRAMETRICS COMPUTED ARE COMPARED AND FALSE AND TRUE TRIADS
C       ARE ACCUMULATED.
C
C   INPUT PARAMETERS
C   ----------------
C
C   IOBJ  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE OBJECT NUMBER.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF OBJECTS.
C
C   K     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF NODES OF THE TREE, MUST BE GREATER THAN M.
C
C   MT    VECTOR OF INTEGERS DIMENSIONED AT LEAST K (UNCHANGED ON
C            OUTPUT).
C         MT(I) IS THE ANCESTOR OF OBJECT I IN THE TREE.
C
C   DMT1  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX T.  MUST BE AT LEAST M.
C
C   DMT2  INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE SECOND DIMENSION OF THE MATRIX T.  MUST BE AT LEAST M.
C
C   T     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMT1, WHOSE SECOND
C            DIMENSION MUST BE AT DMT2, AND WHOSE THIRD DIMENSION MUST
C            AT LEAST M (UNCHANGED ON OUTPUT).
C         T(I,J,K) = 1. IF (IJ)K IS A LEGAL TRIAD, IE. IF THE DISTANCE
C         FROM OBJECT I TO OBJECT J IS SMALLER THAN THE DISTANCE FROM
C         OBJECT J TO OBJECT K AND THE DISTANCE FROM OBJECT I TO OBJECT
C         K.
C
C   DMULT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF THE MATRIX ULT.  MUST BE AT LEAST M.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   ULT   REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMTR, AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST M.
C         ULT(I,J) IS THE ULTRAMETRIC BETWEEN OBJECT I AND OBJECT J.
C
C   TT    REAL SCALAR.
C         NUMBER OF TRUE TRIADS.
C
C   FF    REAL SCALAR.
C         NUMBER OF FALSE TRIADS.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 189.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMULT, DMT1, DMT2
      DIMENSION MT(*), ULT(DMULT,*), T(DMT1,DMT2,*)
C
      IL=IOBJ
      IU=IOBJ
      IF(IOBJ.LT.1.OR.IOBJ.GT.M) THEN
         IL=1
         IU=M
      ENDIF
      FT=0.
      TT=0.
C
C     UPDATES ULTRAMETRIC
C
      DO 10 I=IL,IU
   10    CALL ULTRA(I,M,K,MT,DMULT,ULT)
C
C     COMPUTES NUMBER OF FALSE AND TRUE TRIADS
C
      DO 20 I=IL,IU
         DO 20 J=1,M
            DO 20 L=1,M
               IF(I.NE.J.AND.J.NE.L.AND.L.NE.I) THEN
                  IF(ULT(I,J).LT.ULT(I,L)) THEN
                     TT=TT+T(I,J,L)
                     FT=FT+T(I,L,J)+T(L,J,I)
                  ENDIF
                  IF(ULT(J,L).LT.ULT(I,J)) THEN
                     TT=TT+T(L,J,I)*0.5
                     FT=FT+(T(I,L,J)+T(I,J,L))*0.5
                  ENDIF
               ENDIF
   20 CONTINUE
      RETURN
      END
