      SUBROUTINE TRY1(MM, M, N, A, NDIAGS, NV, DMB, B, XLL, V)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      FINDS THE BEST SET OF BLOCKS, CHANGING ONE AT A TIME
C
C   DESCRIPTION
C   -----------
C
C   1.  THE FIRST BLOCK STARTS WITH THE MINIMUM AND MAXIMUM VALUES FOR
C       EACH VARIABLE.  THEN EACH VARIABLE WITHIN THE BLOCK IS
C       COMPRESSED AND EXPANDED WITH THE LOG LIKELIHOOD OF THE NEW
C       BLOCK DETERMINED EACH TIME.  AFTER THE 5 CHANGES TO EACH
C       VARIABLE IN THE BLOCK, THE BLOCK WITH THE LARGEST
C       LOG-LIKELIHOOD IS RESTORED.  THIS PROCESS IS REPEATED TWICE
C       MORE AFTER STARTING AT THE PREVIOUS BEST BLOCK.  AFTER THE
C       THREE ITERATIONS THE FIRST BLOCK IS COMPLETE.  THE PROCESS
C       REPEATS BY ADDING ON A SECOND BLOCK AND COMPUTING THE
C       LIKELIHOOD OF THE TWO BLOCKS, CHANGING THE SECOND BLOCK UNTIL A
C       MAXIMUM IS REACHED.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (UNCHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C   NDIAGS INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF DIAGRAMS.
C
C   NV    INTEGER VECTOR DIMENSIONED AT LEAST N (UNCHANGED ON OUTPUT).
C         VECTOR DEFINING THE ORDER OF THE VARIABLES.
C
C   DMB   INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         LEADING DIMENSION OF THE MATRIX B.  MUST BE AT LEAST N.
C
C   V     REAL MATRIX DIMENSIONED AT LEAST 2*NDIAGS.
C         WORK MATRIX.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   B     REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMB AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST 2*NDIAGS.
C         THE MATRIX WHICH DEFINES THE BLOCKS.
C
C        B(J,I) IS THE MINIMUM VALUE FOR THE I-TH VARIABLE IN THE J-TH
C                      BLOCK.
C        B(NDIAGS+J,I) IS THE MAXIMUM VALUE FOR THE I-TH VARIABLE IN THE
C                      J-TH BLOCK.
C
C   XLL   REAL SCALAR.
C         THE LOG LIKELIHOOD OF THE FINAL BLOCKS.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 52.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMB
      DIMENSION NV(*), A(MM,*), B(DMB,*), V(*), XX(5)
C
      DO 40 L = 1 , NDIAGS
C
C     GET INITIAL LIKELIHOOD FOR BLOCK
C
         CALL LOB(MM,M,N,A,NDIAGS,DMB,B,XLL,V)
         DO 30 ITER=1,3
            DO 20 JJ=1,N
               J=NV(JJ)
               B1=B(L,J)
               B2=B(NDIAGS+L,J)
               VM=0.
               KM=1
C
C    DETERMINE THE LOG LIKELIHOOD FOR EACH BLOCK KEEPING TRACK OF THE
C    LARGEST
C
               DO 10 K=1,5
                  CALL PICK(K,B1,B2,B(L,J),B(NDIAGS+L,J))
                  CALL LOB(MM,M,N,A,NDIAGS,DMB,B,XX(K),V)
                  IF(V(NDIAGS+L).NE.0) V(NDIAGS+L)=V(L)/V(NDIAGS+L)
                  IF(V(NDIAGS+L).GT.VM) THEN
                     KM=K
                     VM=V(NDIAGS+L)
                  ENDIF
   10          CONTINUE
               B(L,J)=B1
               B(NDIAGS+L,J)=B2
C
C    IF GREATER THAN THE PREVIOUS MAXIMUM LOG LIKELIHOOD, UPDATE
C
               IF(XX(KM).GE.XLL) THEN
                  XLL=XX(KM)
                  CALL PICK(KM,B1,B2,B(L,J),B(NDIAGS+L,J))
               ENDIF
   20       CONTINUE
   30    CONTINUE
   40 CONTINUE
      RETURN
      END
