      SUBROUTINE TWO(MM, M, N, A, CLAB, RLAB, TITLE, DMCOV, COV,
     1               COVLAB, COVTIT, AVE, AVECLB, AVERLB, AVETIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      COMPUTES OVERALL MEAN AND COVARIANCE MATRICES OF A DATA SET WITH
C      APPROPRIATE LABELS FOR THE CASES AND VARIABLES
C
C   DESCRIPTION
C   -----------
C
C   1.  MISSING VALUES ON INPUT ARE DENOTED BY 99999.  THE NON-MISSING
C       VALUES FOR EACH VARIABLE ARE SUMMED AND THE OVERALL MEAN IS
C       COMPUTED.  THEN EACH MISSING VALUE IS REPLACED BY THE MEAN FOR
C       THE RESPECTIVE VARIABLE.
C
C   2.  THE COVARIANCE MATRIX IS THEN CALCULATED.
C
C       SINCE BOTH THE ROW AND COLUMN LABELS FOR THE COVARIANCE MATRIX
C       ARE THE SAME, ONLY ONE SET IS GENERATED.  TO USE THE CLUSTER
C       ROUTINE "OUT" TO PRINT OUT THE COVARIANCE MATRIX, THE SAME
C       VECTOR OF LABELS CAN BE USED FOR THE RLAB AND CLAB ARGUMENTS.
C       THE ROUTINE WILL RETURN BOTH ROW AND COLUMN LABELS FOR THE
C       AVERAGE VECTOR IN ORDER TO ALLOW THE USER TO USE THE CLUSTER
C       ROUTINE "OUT" TO OUTPUT THIS VECTOR WITHOUT ANY OTHER SETUP.
C       SEE PROGRAM "TWO" IN THE SAMPLE FILE UNDER THE HARTIGA ACCOUNT
C       FOR AN EXAMPLE OF USING "OUT" AFTER "TWO".
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N. (CHANGED ON OUTPUT).
C         THE MATRIX OF DATA VALUES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   CLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE VARIABLES.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   DMCOV INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE LEADING DIMENSION OF THE MATRIX COV.  MUST BE AT LEAST N.
C
C   OUTPUT PARAMETERS
C   -----------------
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N.
C         THE MATRIX OF DATA VALUES WITH MISSING VALUES REPLACED.
C
C   COV   REAL MATRIX WHOSE FIRST DIMENSION MUST BE DMCOV AND WHOSE
C            SECOND DIMENSION MUST BE AT LEAST N.
C         THE COVARIANCE MATRIX.
C
C   COVLAB MATRIX OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C         THE LABELS OF THE VARIABLES FOR THE COVARIANCE MATRIX.
C
C   COVTIT 10-CHARACTER VARIABLE.
C         TITLE OF THE COVARIANCE MATRIX.
C
C   AVE   REAL VECTOR WHOSE DIMENSION MUST BE AT LEAST N.
C         THE VECTOR OF VARIABLE AVERAGES.
C
C   AVECLB VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C         THE COLUMN LABELS FOR THE AVERAGE VECTOR.
C
C   AVERLB 4-CHARACTER VARIABLE.
C         THE LABEL OF THE ROW FOR THE AVERAGE VECTOR.  (BLANKED OUT,
C         USED TO COMPLETE THE PARAMETERS TO THE "OUT" ROUTINE IN
C         CLUSTER)
C
C   AVETIT 10-CHARACTER VARIABLE.
C         TITLE OF THE AVERAGE VECTOR.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 69, 72.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMCOV
      DIMENSION COV(DMCOV,*),AVE(*),A(MM,*)
      CHARACTER*4 CLAB(*), RLAB(*), COVLAB(*), AVECLB(*), AVERLB
      CHARACTER*10 TITLE, COVTIT, AVETIT
C
C     COMPUTE COLUMN MEANS AND REPLACE MISSING VALUES
C
      DO 30 J=1,N
         AVE(J)=0.
         XP=0.
         DO 10 I=1,M
            IF(A(I,J).NE.99999.) THEN
               XP=XP+1.
               AVE(J)=AVE(J)+A(I,J)
            ENDIF
   10    CONTINUE
         IF(XP.NE.0.) AVE(J)=AVE(J)/XP
         DO 20 I=1,M
   20       IF(A(I,J).EQ.99999.) A(I,J)=AVE(J)
   30 CONTINUE
C
C     COMPUTE COVARIANCES
C
      DO 60 J=1,N
         DO 60 K=1,J
            COV(J,K)=0.
            DO 50 I=1,M
   50          COV(J,K)=COV(J,K)+(A(I,J)-AVE(J))*(A(I,K)-AVE(K))
            COV(J,K)=COV(J,K)/M
   60 COV(K,J)=COV(J,K)
C
C     LABEL MATRICES
C
      DO 70 J=1,N
         AVECLB(J)=CLAB(J)
   70    COVLAB(J)=CLAB(J)
      AVERLB = '    '
      COVTIT = 'COVARIANCE'
      AVETIT = ' AVERAGES '
      RETURN
      END
