      SUBROUTINE XDPMUP(NU1,NU2,MU1,MU2,PQA,IPQA)
C***BEGIN PROLOGUE  XDPMUP
C***REFER TO  XDLEGF
C***ROUTINES CALLED  XDADJ
C***DATE WRITTEN   820728   (YYMMDD)
C***REVISION DATE  871119   (YYMMDD)
C***CATEGORY NO.  C3a2,C9
C***KEYWORDS  LEGENDRE FUNCTIONS
C***AUTHOR  SMITH, JOHN M. (NBS AND GEORGE MASON UNIVERSITY)
C***PURPOSE  TO COMPUTE THE VALUES OF LEGENDRE FUNCTIONS FOR XDLEGF.
C        SUBROUTINE XDPMUP TRANSFORMS AN ARRAY OF LEGENDRE FUNCTIONS
C        OF THE FIRST KIND OF NEGATIVE ORDER STORED IN ARRAY
C        PQA INTO LEGENDRE FUNCTIONS OF THE FIRST KIND OF
C        POSITIVE ORDER  STORED IN ARRAY PQA. THE ORIGINAL
C        ARRAY IS DESTROYED.
C***REFERENCES  OLVER AND SMITH,J.COMPUT.PHYSICS,51(1983),N0.3,502-518.
C***END PROLOGUE  XDPMUP
      DOUBLE PRECISION DMU,NU,NU1,NU2,PQA,PROD
      DIMENSION PQA(*),IPQA(*)
C***FIRST EXECUTABLE STATEMENT   XDPMUP
      NU=NU1
      MU=MU1
      DMU=DBLE(FLOAT(MU))
      N=IFIX(SNGL(NU2-NU1+.1))+(MU2-MU1)+1
      J=1
      IF(AMOD(SNGL(NU),1.).NE.0.) GO TO 210
  200 IF(DMU.LT.NU+1.D0) GO TO 210
      PQA(J)=0.
      IPQA(J)=0
      J=J+1
      IF(J.GT.N) RETURN
C        INCREMENT EITHER MU OR NU AS APPROPRIATE.
      IF(NU2-NU1.GT..5D0) NU=NU+1.D0
      IF(MU2.GT.MU1) MU=MU+1
      GO TO 200
C
C        TRANSFORM P(-MU,NU,X) TO P(MU,NU,X) USING
C        P(MU,NU,X)=(NU-MU+1)*(NU-MU+2)*...*(NU+MU)*P(-MU,NU,X)*(-1)**MU
C
  210 PROD=1.D0
      IPROD=0
      K=2*MU
      IF(K.EQ.0) GO TO 222
      DO 220 L=1,K
      PROD=PROD*(DMU-NU-DBLE(FLOAT(L)))
  220 CALL XDADJ(PROD,IPROD)
  222 CONTINUE
      DO 240 I=J,N
      IF(MU.EQ.0) GO TO 225
      PQA(I)=PQA(I)*PROD*DBLE(FLOAT((-1)**MU))
      IPQA(I)=IPQA(I)+IPROD
      CALL XDADJ(PQA(I),IPQA(I))
  225 IF(NU2-NU1.GT..5D0) GO TO 230
      PROD=(DMU-NU)*PROD*(-DMU-NU-1.D0)
      CALL XDADJ(PROD,IPROD)
      MU=MU+1
      DMU=DMU+1.D0
      GO TO 240
  230 PROD=PROD*(-DMU-NU-1.D0)/(DMU-NU-1.D0)
      CALL XDADJ(PROD,IPROD)
      NU=NU+1.D0
  240 CONTINUE
      RETURN
      END
