      SUBROUTINE XSPMU(NU1,NU2,MU1,MU2,THETA,X,SX,ID,PQA,IPQA)
C***BEGIN PROLOGUE  XSPMU
C***REFER TO  XSLEGF
C***ROUTINES CALLED  XSADD, XSADJ, XSPQNU
C***DATE WRITTEN   820728   (YYMMDD)
C***REVISION DATE  871119   (YYMMDD)
C***CATEGORY NO.  C3a2,C9
C***KEYWORDS  LEGENDRE FUNCTIONS
C***AUTHOR  SMITH, JOHN M. (NBS AND GEORGE MASON UNIVERSITY)
C***PURPOSE  TO COMPUTE THE VALUES OF LEGENDRE FUNCTIONS FOR XSLEGF.
C METHOD: BACKWARD MU-WISE RECURRENCE FOR P(-MU,NU,X) FOR FIXED NU TO
C         OBTAIN P(-MU2,NU1,X),P(-(MU2-1),NU1,X),....,P(-MU1,NU1,X)
C         AND STORE IN ASCENDING MU ORDER.
C***REFERENCES  OLVER AND SMITH,J.COMPUT.PHYSICS,51(1983),N0.3,502-518.
C***END PROLOGUE  XSPMU
      REAL PQA,NU1,NU2,P0,X,SX,THETA,X1,X2
      DIMENSION PQA(*),IPQA(*)
C
C        CALL XSPQNU TO OBTAIN P(-MU2,NU,X)
C
C***FIRST EXECUTABLE STATEMENT   XSPMU
      CALL XSPQNU(NU1,NU2,MU2,THETA,ID,PQA,IPQA)
      P0=PQA(1)
      IP0=IPQA(1)
      MU=MU2-1
C
C        CALL XSPQNU TO OBTAIN P(-MU2-1,NU,X)
C
      CALL XSPQNU(NU1,NU2,MU,THETA,ID,PQA,IPQA)
      N=MU2-MU1+1
      PQA(N)=P0
      IPQA(N)=IP0
      IF(N.EQ.1) GO TO 300
      PQA(N-1)=PQA(1)
      IPQA(N-1)=IPQA(1)
      IF(N.EQ.2) GO TO 300
      J=N-2
  290 CONTINUE
C
C        BACKWARD RECURRENCE IN MU TO OBTAIN
C              P(-MU2,NU1,X),P(-(MU2-1),NU1,X),....P(-MU1,NU1,X)
C              USING
C              (NU-MU)*(NU+MU+1.)*P(-(MU+1),NU,X)=
C                2.*MU*X*SQRT((1./(1.-X**2))*P(-MU,NU,X)-P(-(MU-1),NU,X)
C
      X1=2.*(FLOAT(MU))*X*SX*PQA(J+1)
      X2=-(NU1-(FLOAT(MU)))*(NU1+(FLOAT(MU))+1.)*PQA(J+2)
      CALL XSADD(X1,IPQA(J+1),X2,IPQA(J+2),PQA(J),IPQA(J))
      CALL XSADJ(PQA(J),IPQA(J))
      IF(J.EQ.1) GO TO 300
      J=J-1
      MU=MU-1
      GO TO 290
  300 RETURN
      END
