      COMPLEX FUNCTION C9LGMC(ZIN)
C***BEGIN PROLOGUE  C9LGMC
C***DATE WRITTEN   780401   (YYMMDD)
C***REVISION DATE  820801   (YYMMDD)
C***CATEGORY NO.  C7A
C***KEYWORDS  COMPLETE GAMMA FUNCTION,COMPLEX,CORRECTION TERM,
C             GAMMA FUNCTION,LOGARITHM,SPECIAL FUNCTION
C***AUTHOR  FULLERTON, W., (LANL)
C***PURPOSE  Computes the LOG GAMMA correction term for most Z so that
C            CLOG(CGAMMA(Z)) = 0.5*ALOG(2.*PI) + (Z-0.5)*CLOG(Z) - Z
C            + C9LGMC(Z)
C***DESCRIPTION
C
C Compute the LOG GAMMA correction term for large CABS(Z) when REAL(Z)
C .GE. 0.0 and for large ABS(AIMAG(Y)) when REAL(Z) .LT. 0.0.  We find
C C9LGMC so that
C   CLOG((Z)) = 0.5*ALOG(2.*PI) + (Z-0.5)*CLOG(Z) - Z + C9LGMC(Z)
C***REFERENCES  (NONE)
C***ROUTINES CALLED  R1MACH,XERROR
C***END PROLOGUE  C9LGMC
      COMPLEX ZIN, Z, Z2INV
      DIMENSION BERN(11)
      DATA BERN( 1) /    .08333333333 3333333E0   /
      DATA BERN( 2) /   -.002777777777 7777778E0  /
      DATA BERN( 3) /    .0007936507936 5079365E0 /
      DATA BERN( 4) /   -.0005952380952 3809524E0 /
      DATA BERN( 5) /    .0008417508417 5084175E0 /
      DATA BERN( 6) /   -.001917526917 5269175E0  /
      DATA BERN( 7) /    .006410256410 2564103E0  /
      DATA BERN( 8) /   -.02955065359 4771242E0   /
      DATA BERN( 9) /    .1796443723 6883057E0    /
      DATA BERN(10) /  -1.392432216 9059011E0     /
      DATA BERN(11) /  13.40286404 4168392E0      /
      DATA NTERM, BOUND, XBIG, XMAX / 0, 3*0.0 /
C***FIRST EXECUTABLE STATEMENT  C9LGMC
      IF (NTERM.NE.0) GO TO 10
C
      NTERM = -0.30*ALOG(R1MACH(3))
      BOUND = 0.1170*FLOAT(NTERM)*
     1  (0.1*R1MACH(3))**(-1./(2.*FLOAT(NTERM)-1.))
      XBIG = 1.0/SQRT(R1MACH(3))
      XMAX = EXP (AMIN1(ALOG(R1MACH(2)/12.0), -ALOG(12.*R1MACH(1))) )
C
 10   Z = ZIN
      X = REAL (Z)
      Y = AIMAG(Z)
      CABSZ = CABS(Z)
C
      IF (X.LT.0.0 .AND. ABS(Y).LT.BOUND) CALL XERROR (  'C9LGMC  C9LGMC
     1 NOT VALID FOR NEGATIVE REAL(Z) AND SMALL ABS(AIMAG(Z))', 69, 2,2)
      IF (CABSZ.LT.BOUND) CALL XERROR ( 'C9LGMC  C9LGMC NOT VALID FOR SM
     1ALL CABS(Z)', 42, 3, 2)
C
      IF (CABSZ.GE.XMAX) GO TO 50
C
      IF (CABSZ.GE.XBIG) C9LGMC = 1.0/(12.0*Z)
      IF (CABSZ.GE.XBIG) RETURN
C
      Z2INV = 1.0/Z**2
      C9LGMC = (0.0, 0.0)
      DO 40 I=1,NTERM
        NDX = NTERM + 1 - I
        C9LGMC = BERN(NDX) + C9LGMC*Z2INV
 40   CONTINUE
C
      C9LGMC = C9LGMC/Z
      RETURN
C
 50   C9LGMC = (0.0, 0.0)
      CALL XERROR ( 'C9LGMC  Z SO BIG C9LGMC UNDERFLOWS', 34, 1, 1)
      RETURN
C
      END
