      DOUBLE PRECISION FUNCTION DBETA(A,B)
C***BEGIN PROLOGUE  DBETA
C***DATE WRITTEN   770601   (YYMMDD)
C***REVISION DATE  820801   (YYMMDD)
C***CATEGORY NO.  C7B
C***KEYWORDS  BETA,BETA FUNCTION,COMPLETE BETA FUNCTION,
C             DOUBLE PRECISION,SPECIAL FUNCTION
C***AUTHOR  FULLERTON, W., (LANL)
C***PURPOSE  Computes the d.p. complete Beta function.
C***DESCRIPTION
C
C DBETA(A,B) calculates the double precision complete beta function
C for double precision arguments A and B.
C***REFERENCES  (NONE)
C***ROUTINES CALLED  D1MACH,DGAMLM,DGAMMA,DLBETA,XERROR
C***END PROLOGUE  DBETA
      EXTERNAL DGAMMA
      DOUBLE PRECISION A, B, ALNSML, XMAX, XMIN, DLBETA, DGAMMA, D1MACH
      DATA XMAX, ALNSML / 0.D0, 0.D0 /
C***FIRST EXECUTABLE STATEMENT  DBETA
      IF (XMAX.NE.0.D0) GO TO 10
      CALL DGAMLM (XMIN, XMAX)
      ALNSML = DLOG (D1MACH(1))
C
 10   IF (A.LE.0.D0 .OR. B.LE.0.D0) CALL XERROR ( 'DBETA   BOTH ARGUMENT
     1S MUST BE GT 0', 35, 2, 2)
C
      IF (A+B.LT.XMAX) DBETA = DGAMMA(A)*DGAMMA(B)/DGAMMA(A+B)
      IF (A+B.LT.XMAX) RETURN
C
      DBETA = DLBETA (A, B)
      IF (DBETA.LT.ALNSML) GO TO 20
      DBETA = DEXP (DBETA)
      RETURN
C
 20   DBETA = 0.D0
      CALL XERROR ( 'DBETA   A AND/OR B SO BIG BETA UNDERFLOWS', 41, 1,
     1  1)
      RETURN
C
      END
