      DOUBLE PRECISION FUNCTION DLBETA(A,B)
C***BEGIN PROLOGUE  DLBETA
C***DATE WRITTEN   770701   (YYMMDD)
C***REVISION DATE  820801   (YYMMDD)
C***CATEGORY NO.  C7B
C***KEYWORDS  BETA,BETA FUNCTION,COMPLETE BETA FUNCTION,
C             DOUBLE PRECISION,LOGARITHM,SPECIAL FUNCTION
C***AUTHOR  FULLERTON, W., (LANL)
C***PURPOSE  Computes the  d.p. natural logarithm of the complete Beta
C            function
C***DESCRIPTION
C
C DLBETA(A,B) calculates the double precision natural logarithm of
C the complete beta function for double precision arguments
C A and B.
C***REFERENCES  (NONE)
C***ROUTINES CALLED  D9LGMC,DGAMMA,DLNGAM,DLNREL,XERROR
C***END PROLOGUE  DLBETA
      EXTERNAL DGAMMA
      DOUBLE PRECISION A, B, P, Q, CORR, SQ2PIL, D9LGMC, DGAMMA, DLNGAM,
     1  DLNREL
      DATA SQ2PIL / 0.9189385332 0467274178 0329736405 62 D0 /
C***FIRST EXECUTABLE STATEMENT  DLBETA
      P = DMIN1 (A, B)
      Q = DMAX1 (A, B)
C
      IF (P.LE.0.D0) CALL XERROR ( 'DLBETA  BOTH ARGUMENTS MUST BE GT ZE
     1RO', 38, 1, 2)
C
      IF (P.GE.10.D0) GO TO 30
      IF (Q.GE.10.D0) GO TO 20
C
C P AND Q ARE SMALL.
C
      DLBETA = DLOG (DGAMMA(P) * (DGAMMA(Q)/DGAMMA(P+Q)) )
      RETURN
C
C P IS SMALL, BUT Q IS BIG.
C
 20   CORR = D9LGMC(Q) - D9LGMC(P+Q)
      DLBETA = DLNGAM(P) + CORR + P - P*DLOG(P+Q)
     1  + (Q-0.5D0)*DLNREL(-P/(P+Q))
      RETURN
C
C P AND Q ARE BIG.
C
 30   CORR = D9LGMC(P) + D9LGMC(Q) - D9LGMC(P+Q)
      DLBETA = -0.5D0*DLOG(Q) + SQ2PIL + CORR + (P-0.5D0)*DLOG(P/(P+Q))
     1  + Q*DLNREL(-P/(P+Q))
      RETURN
C
      END
