*DMPREC
      DOUBLE PRECISION FUNCTION DMPREC()
C***BEGIN PROLOGUE  DPREC
C***REFER TO  DODR,DODRC
C***ROUTINES CALLED  (NONE)
C***DATE WRITTEN   860529   (YYMMDD)
C***REVISION DATE  870204   (YYMMDD)
C***CATEGORY NO.  G2E,I1B1
C***KEYWORDS  ORTHOGONAL DISTANCE REGRESSION,
C             NONLINEAR LEAST SQUARES,
C             ERRORS IN VARIABLES
C***AUTHOR  BOGGS, PAUL T.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, GAITHERSBURG, MD 20899
C           BYRD, RICHARD H.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C           DONALDSON, JANET R.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C           SCHNABEL, ROBERT B.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C             AND
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C***PURPOSE  DETERMIND MACHINE PRECISION FOR TARGET MACHINE AND COMPILER
C            ASSUMING FLOATING-POINT NUMBERS ARE REPRESENTED IN THE
C            T-DIGIT, BASE-B FORM
C               SIGN (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
C            WHERE 0 .LE. X(I) .LT. B FOR I=1,...,T,
C                  0 .LT. X(1), AND EMIN .LE. E .LE. EMAX.
C***END PROLOGUE  DPREC
C
C  VARIABLE DECLARATIONS (ALPHABETICALLY)
C
      DOUBLE PRECISION B
C        THE BASE OF THE TARGET MACHINE.
C        (MAY BE DEFINED USING I1MACH(10).)
      INTEGER TD
C        THE NUMBER OF BASE-B DIGITS IN DOUBLE PRECISION.
C        (MAY BE DEFINED USING I1MACH(14).)
      INTEGER TS
C        THE NUMBER OF BASE-B DIGITS IN SINGLE PRECISION.
C        (MAY BE DEFINED USING I1MACH(11).)
C
C
C  TO ALTER THIS FUNCTION FOR A PARTICULAR TARGET MACHINE,
C  THE DESIRED SET OF DATA STATEMENTS SHOULD BE ACTIVATED BY
C  REMOVING THE C FROM COLUMN 1.
C
C
C  MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM.
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  60 /
C
C  MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM AND
C                        THE BURROUGHS 6700/7700 SYSTEMS.
C
C     DATA B  /   8 /
C     DATA TS /  13 /
C     DATA TD /  26 /
C
C  MACHINE CONSTANTS FOR THE CDC 6000/7000 AND
C                        THE CYBER 170/180/200 SERIES.
C
C     DATA B  /   2 /
C     DATA TS /  48 /
C     DATA TD /  96 /
C
C  MACHINE CONSTANTS FOR THE CRAY 1
C
C     DATA B  /   2 /
C     DATA TS /  47 /
C     DATA TD /  94 /
C
C  MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200
C
C     DATA B  /  16 /
C     DATA TS /   6 /
C     DATA TD /  14 /
C
C  MACHINE CONSTANTS FOR THE HARRIS SLASH 6 AND SLASH 7
C
C     DATA B  /   2 /
C     DATA TS /  23 /
C     DATA TD /  38 /
C
C  MACHINE CONSTANTS FOR THE HONEYWELL DPS 8/70
C
C     DATA B  /   2 /
C     DATA TS /  27 /
C     DATA TD /  63 /
C
C  MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
C     THE XEROX SIGMA 5/7/9 AND THE SEL SYSTEMS 85/86.
C
C     DATA B  /  16 /
C     DATA TS /   6 /
C     DATA TD /  14 /
C
C  MACHINE CONSTANTS FOR THE IBM PC USING PROFORT.
C
C     DATA B  /   2 /
C     DATA TS /  23 /
C     DATA TD /  52 /
C
C  MACHINE CONSTANTS FOR THE PERKIN-ELMER 3230
C
C     DATA B  /  16 /
C     DATA TS /   6 /
C     DATA TD /  14 /
C
C  MACHINE CONSTANTS FOR THE INTERDATA 8/32 WITH THE UNIX SYSTEM
C     FORTRAN 77 COMPILER.
C
C     DATA B  /  16 /
C     DATA TS /   6 /
C     DATA TD /  14 /
C
C  MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR).
C
C     DATA B  /   2 /
C     DATA TS /  27 /
C     DATA TD /  54 /
C
C  MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR).
C
C     DATA B  /   2 /
C     DATA TS /  27 /
C     DATA TD /  62 /
C
C  MACHINE CONSTANTS FOR PDP-11 FORTRAN'S SUPPORTING
C     32-BIT INTEGER ARITHMETIC.
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  56 /
C
C  MACHINE CONSTANTS FOR PDP-11 FORTRAN'S SUPPORTING
C     16-BIT INTEGER ARITHMETIC.
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  56 /
C
C  MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES.
C
C     DATA B  /   2 /
C     DATA TS /  27 /
C     DATA TD /  60 /
C
C  MACHINE CONSTANTS FOR THE VAX-11 WITH FORTRAN IV-PLUS
C     COMPILER AND FOR THE VAX/VMS VERSION 2.2
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  56 /
C
C  MACHINE CONSTANTS FOR THE VAX/VMS V4 SYSTEM USING D_FLOATING
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  56 /
C
C  MACHINE CONSTANTS FOR THE VAX/VMS V4 SYSTEM USING G_FLOATING
C
C     DATA B  /   2 /
C     DATA TS /  24 /
C     DATA TD /  53 /
C
C
C***FIRST EXECUTABLE STATEMENT  DPREC
C
C
      B = I1MACH(10)
      TD = I1MACH(14)
      TS = I1MACH(11)
C
      DMPREC = B ** (1-TD)
C
      RETURN
C
      END
