*SETAF
      SUBROUTINE SETAF
     +   (FUN,N,NP,M,XPLUSD,LDXPD,BETA,ETA,NETA,EPSMAC,
     +   NROW,PARTMP,PVTEMP,IFLAG)
C***BEGIN PROLOGUE  SETAF
C***REFER TO  SODR,SODRC
C***ROUTINES CALLED  (NONE)
C***DATE WRITTEN   860529   (YYMMDD)
C***REVISION DATE  870204   (YYMMDD)
C***CATEGORY NO.  G2E,I1B1
C***KEYWORDS  ORTHOGONAL DISTANCE REGRESSION,
C             NONLINEAR LEAST SQUARES,
C             ERRORS IN VARIABLES
C***AUTHOR  BOGGS, PAUL T.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, GAITHERSBURG, MD 20899
C           BYRD, RICHARD H.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C           DONALDSON, JANET R.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C           SCHNABEL, ROBERT B.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C             AND
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C***PURPOSE  COMPUTE NOISE AND NUMBER OF GOOD DIGITS IN FUNCTION RESULTS
C            (THIS ROUTINE IS MODELED AFTER STARPAC SUBROUTINE ETAFUN)
C***END PROLOGUE  SETAF
C
C  EXTERNALS
C
      EXTERNAL FUN
C        THE NAME OF USER-SUPPLIED ROUTINE FOR COMPUTING THE MODEL.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE SECTION V.B,
C        ARGUMENT FUN.)
C
C  VARIABLE DECLARATIONS (ALPHABETICALLY)
C
      REAL A
C        PARAMETERS OF THE FIT.
      REAL B
C        PARAMETERS OF THE FIT.
      REAL BETA(NP)
C        THE FUNCTION PARAMETERS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      REAL EPSMAC
C        THE VALUE OF MACHINE PRECISION.
      REAL ETA
C        THE NOISE IN THE MODEL RESULTS.
      REAL FAC
C        A FACTOR USED IN THE COMPUTATIONS.
      INTEGER I
C        AN INDEXING VARIABLE.
      INTEGER IFLAG
C        AN INDICATOR VARIABLE, USED PRIMARILY TO DESIGNATE THAT THE
C        USER WISHES THE COMPUTATIONS STOPPED.
      REAL J
C        THE VALUE FLOAT(I-3).
      INTEGER K
C        AN INDEX VARIABLE.
      INTEGER LDXPD
C        THE LEADING DIMENSION OF ARRAY XPLUSD.
      INTEGER M
C        THE NUMBER OF COLUMNS OF DATA IN THE INDEPENDENT VARIABLE.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER N
C        THE NUMBER OF OBSERVATIONS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER NETA
C        THE NUMBER OF ACCURATE DIGITS IN THE MODEL RESULTS.
      INTEGER NP
C        THE NUMBER OF FUNCTION PARAMETERS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER NROW
C        THE NUMBER OF THE ROW AT WHICH THE DERIVATIVE IS TO BE CHECKED.
      REAL ONE
C        THE VALUE 1.0E0.
      REAL P1
C        THE VALUE 0.1E0.
      REAL P2
C        THE VALUE 0.2E0.
      REAL PARTMP(NP)
C        MODIFIED MODEL PARAMETERS
      REAL PVTEMP(N)
C        PREDICTED VALUES
      REAL RSS(5)
C        THE RESIDUAL SUM OF SQUARES FOR EACH VALUE OF J.
      REAL RSSSM
C        THE SUM OF THE RESIDUAL SUM OF SQUARES FOR EACH SET OF
C        PARAMETER VALUES.
      REAL RSSSMJ
C        THE SUM OF THE RESIDUAL SUM OF SQUARES TIMES J FOR EACH
C        SET OF PARAMETER VALUES.
      REAL SQRTMP
C        THE SQUARE ROOT OF MACHINE PRECISION (EPSMAC).
      REAL XPLUSD(LDXPD,M)
C        THE ARRAY X + DELTA.
      REAL ZERO
C        THE VALUE 0.0E0.
C
C
      DATA ZERO,P1,P2,ONE/0.0E0,0.1E0,0.2E0,1.0E0/
C
C
C***FIRST EXECUTABLE STATEMENT  SETAF
C
C
      SQRTMP = SQRT(EPSMAC)
      RSSSM = ZERO
      RSSSMJ = ZERO
      DO 20 I=1,5
         J = I-3
         DO 10 K=1,NP
            PARTMP(K) = BETA(K)*(ONE+J*SQRTMP)
   10    CONTINUE
         IFLAG = 0
         CALL FUN(N,NP,M,PARTMP,XPLUSD,LDXPD,PVTEMP,IFLAG)
         IF (IFLAG.LT.0) THEN
            RETURN
         END IF
C
         RSS(I) = PVTEMP(NROW)
C
         RSSSM = RSSSM + RSS(I)
         RSSSMJ = RSSSMJ + J*RSS(I)
   20 CONTINUE
      A = P2*RSSSM
      B = P1*RSSSMJ
      IF (RSS(3).NE.ZERO) THEN
         FAC = ONE/ABS(RSS(3))
      ELSE
         FAC = ONE
      END IF
      DO 30 I=1,5
         J = I-3
         RSS(I) = ABS((RSS(I)-(A+J*B))*FAC)
   30 CONTINUE
      ETA = MAX(RSS(1),RSS(2),RSS(3),RSS(4),RSS(5),EPSMAC)
      NETA = INT(-LOG10(ETA))
C
      RETURN
      END
