*SODCHK
      SUBROUTINE SODCHK
     +   (N,NP,M,
     +   IFIXB,
     +   LDX,LDIFX,LDSCLD,LDRHO,
     +   LWORK,LWKMN,LIWORK,LIWKMN,
     +   SCLD,SCLB,W,RHO,
     +   EPSMAC,INFO)
C***BEGIN PROLOGUE  SODCHK
C***REFER TO  SODR,SODRC
C***ROUTINES CALLED  (NONE)
C***DATE WRITTEN   860529   (YYMMDD)
C***REVISION DATE  870204   (YYMMDD)
C***CATEGORY NO.  G2E,I1B1
C***KEYWORDS  ORTHOGONAL DISTANCE REGRESSION,
C             NONLINEAR LEAST SQUARES,
C             ERRORS IN VARIABLES
C***AUTHOR  BOGGS, PAUL T.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, GAITHERSBURG, MD 20899
C           BYRD, RICHARD H.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C           DONALDSON, JANET R.
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C           SCHNABEL, ROBERT B.
C             DEPARTMENT OF COMPUTER SCIENCE
C             UNIVERSITY OF COLORADO, BOULDER, CO 80309
C             AND
C             OPTIMIZATION GROUP/SCIENTIFIC COMPUTING DIVISION
C             NATIONAL BUREAU OF STANDARDS, BOULDER, CO 80303-3328
C***PURPOSE  CHECK INPUT PARAMETERS, INDICATING ERRORS FOUND USING
C            NONZERO VALUES OF ARGUMENT INFO AS DESCRIBED IN THE
C            PROLOGUES FOR SODR AND SODRC.
C***END PROLOGUE  SODCHK
C
C  VARIABLE DECLARATIONS (ALPHABETICALLY)
C
      REAL EPSMAC
C        THE VALUE OF MACHINE PRECISION.
      INTEGER I
C        AN INDEXING VARIABLE.
      INTEGER IFIXB(NP)
C        THE INDICATOR VALUES USED TO DESIGNATE WHETHER THE INDIVIDUAL
C        ELEMENTS OF BETA ARE FIXED AT THEIR INPUT VALUES OR NOT.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER INFO
C        AN INDICATOR VARIABLE, USED PRIMARILY TO DESIGNATE WHY THE
C        COMPUTATIONS WERE STOPPED.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER J
C        AN INDEXING VARIABLE.
      INTEGER K
C        AN INDEXING VARIABLE.
      INTEGER LAST
C        THE LAST ROW OF THE ARRAY TO BE ACCESSED.
      INTEGER LDIFX
C        THE LEADING DIMENSION OF ARRAY IFIXX.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER LDRHO
C        THE LEADING DIMENSION OF ARRAY RHO.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER LDSCLD
C        THE LEADING DIMENSION OF ARRAY SCLD.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER LDX
C        THE LEADING DIMENSION OF ARRAY X.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER LIWKMN
C        THE MINIMUM ACCEPTABLE LENGTH OF ARRAY IWORK.
      INTEGER LIWORK
C        THE LENGTH OF VECTOR IWORK.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER LWKMN
C        THE MINIMUM ACCEPTABLE LENGTH OF ARRAY WORK.
      INTEGER LWORK
C        THE LENGTH OF VECTOR WORK.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER M
C        THE NUMBER OF COLUMNS OF DATA IN THE INDEPENDENT VARIABLE.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER MAXGDX
C        THE LARGEST NUMBER OF GOOD DIGITS IT IS REASONABLE TO SPECIFY.
      INTEGER N
C        THE NUMBER OF OBSERVATIONS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER NNZW
C        THE NUMBER OF NON ZERO OBSERVATIONAL ERROR WEIGHTS.
      INTEGER NP
C        THE NUMBER OF FUNCTION PARAMETERS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      INTEGER NPP
C        THE NUMBER OF FUNCTION PARAMETERS ACTUALLY BEING ESTIMATED.
      REAL RHO(LDRHO,M)
C        THE DELTA WEIGHTS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      REAL SCLB(NP)
C        THE SCALE OF EACH BETA.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      REAL SCLD(LDSCLD,M)
C        THE SCALE OF EACH DELTA.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      REAL W(N)
C        THE OBSERVATIONAL ERROR WEIGHTS.
C        (FOR DETAILS, SEE ODRPACK REFERENCE GUIDE.)
      REAL ZERO
C          THE VALUE 0.0E0.
C
C
      DATA ZERO /0.0E0/
C
C
C***FIRST EXECUTABLE STATEMENT  SODCHK
C
C
      MAXGDX = -LOG10(EPSMAC)
C
C  FIND ACTUAL NUMBER OF PARAMETERS BEING ESTIMATED
C
      IF (NP.LE.0 .OR. IFIXB(1).LT.0) THEN
         NPP = NP
      ELSE
         NPP = 0
         DO 10 K=1,NP
            IF (IFIXB(K).NE.0) THEN
               NPP = NPP + 1
            END IF
   10    CONTINUE
      END IF
C
C  CHECK PROBLEM SPECIFICATION PARAMETERS
C
      IF (N.LE.0 .OR. M.LE.0 .OR. NPP.LE.0 .OR. NPP.GT.N) THEN
         INFO = 10000
         IF (N.LE.0) THEN
            INFO = INFO + 1000
         END IF
         IF (M.LE.0) THEN
            INFO = INFO + 100
         END IF
         IF (NPP.LE.0 .OR. NPP.GT.N) THEN
            INFO = INFO + 10
         END IF
         RETURN
      END IF
C
C  CHECK DIMENSION SPECIFICATION PARAMETERS
C
      IF (LDX.LT.N .OR.
     +   (LDIFX.NE.1 .AND. LDIFX.LT.N) .OR.
     +   (LDSCLD.NE.1 .AND. LDSCLD.LT.N) .OR.
     +   (LDRHO.NE.1 .AND. LDRHO.LT.N) .OR.
     +   LWORK.LT.LWKMN .OR. LIWORK.LT.LIWKMN) THEN
         INFO = 20000
         IF (LDX.LT.N) THEN
            INFO = INFO + 1000
         END IF
         IF (LDIFX.NE.1 .AND. LDIFX.LT.N) THEN
            INFO = INFO + 100
         END IF
         IF (LDSCLD.NE.1 .AND. LDSCLD.LT.N) THEN
            INFO = INFO + 200
         END IF
         IF (LDRHO.NE.1 .AND. LDRHO.LT.N) THEN
            INFO = INFO + 400
         END IF
         IF (LWORK.LT.LWKMN) THEN
            INFO = INFO + 10
         END IF
         IF (LIWORK.LT.LIWKMN) THEN
            INFO = INFO + 1
         END IF
         RETURN
      END IF
C
C  CHECK DELTA SCALING
C
      IF (SCLD(1,1).GT.0) THEN
         DO 30 J=1,M
            IF (LDSCLD.GE.N) THEN
               LAST = N
            ELSE
               LAST = 1
            END IF
            DO 20 I=1,LAST
               IF (SCLD(I,J).LE.0) THEN
                  INFO = 31000
                  GO TO 40
               END IF
   20       CONTINUE
   30    CONTINUE
      END IF
C
C  CHECK BETA SCALING
C
   40 IF (SCLB(1).GT.0) THEN
         DO 50 K=1,NP
            IF (SCLB(K).LE.0) THEN
               IF (INFO.EQ.0) THEN
                  INFO = 30100
               ELSE
                  INFO = INFO + 100
               END IF
               GO TO 60
            END IF
   50    CONTINUE
      END IF
C
C  CHECK OBSERVATIONAL ERROR WEIGHTS IF INDIVIDUALLY SPECIFIED
C
   60 IF (W(1).GE.ZERO) THEN
         NNZW = 0
         DO 70 I=2,N
            IF (W(I).LT.ZERO) THEN
               IF (INFO.EQ.0) THEN
                  INFO = 30010
               ELSE
                  INFO = INFO + 10
               END IF
               GO TO 80
            ELSE IF (W(I).GT.ZERO) THEN
               NNZW = NNZW + 1
            END IF
   70    CONTINUE
         IF (NNZW.LT.NPP) THEN
            IF (INFO.EQ.0) THEN
               INFO = 30020
            ELSE
               INFO = INFO + 20
            END IF
         END IF
      END IF
C
C  CHECK DELTA WEIGHTS IF INDIVIDUALLY SPECIFIED
C
   80 IF (RHO(1,1).GE.ZERO) THEN
         DO 100 J=1,M
            IF (LDRHO.GE.N) THEN
               LAST = N
            ELSE
               LAST = 1
            END IF
            DO 90 I=1,LAST
               IF (RHO(I,J).LE.ZERO) THEN
                  IF (INFO.EQ.0) THEN
                     INFO = 30001
                  ELSE
                     INFO = INFO + 1
                  END IF
                  GO TO 110
               END IF
   90       CONTINUE
  100    CONTINUE
      END IF
C
  110 RETURN
C
      END
