      SUBROUTINE SUBFOR ( W, IPIVOT, NROW, LAST, B, X )
C  CARRIES OUT THE FORWARD PASS OF SUBSTITUTION FOR THE CURRENT BLOCK,
C  I.E., THE ACTION ON THE RIGHT SIDE CORRESPONDING TO THE ELIMINATION
C  CARRIED OUT IN  F A C T R B  FOR THIS BLOCK.
C     AT THE END, X(J) CONTAINS THE RIGHT SIDE OF THE TRANSFORMED
C  IPIVOT(J)-TH EQUATION IN THIS BLOCK, J=1,...,NROW. THEN, SINCE
C  FOR I=1,...,NROW-LAST, B(NROW+I) IS GOING TO BE USED AS THE RIGHT
C  SIDE OF EQUATION  I  IN THE NEXT BLOCK (SHIFTED OVER THERE FROM
C  THIS BLOCK DURING FACTORIZATION), IT IS SET EQUAL TO X(LAST+I) HERE.
C
C***REVISION HISTORY  (YYMMDD)
C   000330  Modified array declarations.  (JEC)
C
C PARAMETERS
C    W, IPIVOT, NROW, LAST  ARE AS ON RETURN FROM FACTRB.
C    B(J)   IS EXPECTED TO CONTAIN, ON INPUT, THE RIGHT SIDE OF J-TH
C           EQUATION FOR THIS BLOCK, J=1,...,NROW.
C    B(NROW+J)   CONTAINS, ON OUTPUT, THE APPROPRIATELY MODIFIED RIGHT
C           SIDE FOR EQUATION J IN NEXT BLOCK, J=1,...,NROW-LAST.
C    X(J)   CONTAINS, ON OUTPUT, THE APPROPRIATELY MODIFIED RIGHT
C           SIDE OF EQUATION IPIVOT(J) IN THIS BLOCK, J=1,...,LAST (AND
C           EVEN FOR J=LAST+1,...,NROW).
C
      INTEGER IPIVOT(NROW), IP,JMAX,K
C     DIMENSION B(NROW + NROW-LAST)
      REAL W(NROW,LAST),B(*),X(NROW)
      IP = IPIVOT(1)
      X(1) = B(IP)
      IF (NROW .EQ. 1)                  GO TO 99
      DO 15 K=2,NROW
         IP = IPIVOT(K)
         JMAX = AMIN0(K-1,LAST)
         SUM = 0.
         DO 14 J=1,JMAX
   14       SUM = W(IP,J)*X(J) + SUM
   15    X(K) = B(IP) - SUM
C
C     TRANSFER MODIFIED RIGHT SIDES OF EQUATIONS IPIVOT(LAST+1),...,
C     IPIVOT(NROW) TO NEXT BLOCK.
      NROWML = NROW - LAST
      IF (NROWML .EQ. 0)                GO TO 99
      LASTP1 = LAST+1
      DO 25 K=LASTP1,NROW
   25    B(NROWML+K) = X(K)
   99                                   RETURN
      END
