C
C   DRIVER FOR TESTING CMLIB ROUTINES
C     DQRANK
C     DQRLSS
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(1)
      CALL XERMAX(1000)
C   TEST DQRANK DQRLSS
      CALL DQRLQX(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY DQRLSS
     1  HAS FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- SUBLIBRARY DQRLSS PASSED ALL TESTS ----- ')
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE DQRLQX(KPRINT,IPASS)
C   QUICK TEST PROGRAM FOR DQRANK AND DQRLSS
C
      DOUBLE PRECISION A(3,2),X(2),B(3),QRAUX(3),WORK(3)
      DOUBLE PRECISION EPS,TOL,D2MACH
      INTEGER JPVT(3),KPRINT,IPASS
      COMMON/UNIT/LUN
C
C   SOLVE AX=B IN LEAST SQUARES SENSE
C    A IS 3X2, B IS 3X1 AND X IS 2X1
C
      A(1,1)=1.
      A(1,2)=2.
      A(2,1)=3.
      A(2,2)=4.
      A(3,1)=5.
      A(3,2)=6.
      IPASS=1
      TOL=D2MACH(4)*100.
      CALL DQRANK(A,3,3,2,TOL,KR,JPVT,QRAUX,WORK)
      IF(KPRINT.GT.2) THEN
       WRITE(LUN,100) KR
      ENDIF
  100 FORMAT(' RANK=',I4)
      B(1)=1.
      B(2)=2.
      B(3)=3.
      CALL DQRLSS(A,3,3,2,KR,B,X,B,JPVT,QRAUX)
C  SOLUTION SHOULD BE 0., 0.5
      EPS = SQRT(D2MACH(4))
      IF(ABS(X(1)).LT.EPS .AND. ABS(X(2)-0.50D0).LT.EPS) THEN
          IF(KPRINT.GE.2) THEN
            WRITE(LUN,*) 'TEST FOR SUBROUTINE DQRANK PASSED'
            WRITE(LUN,*) 'TEST FOR SUBROUTINE DQRLSS PASSED'
            IPASS=1
          ENDIF
      ELSE
          IF(KPRINT.GE.1) THEN
            WRITE(LUN,*) 'WARNING -- TEST FOR SUBROUTINE DQRLSS FAILED'
            IPASS=2
            WRITE(LUN,101) X(1),X(2),B(1),B(2),B(3)
  101    FORMAT(' EXACT SOLUTION IS   0.0E0  0.50E0 ' //
     *          ' COMPUTED SOLUTION IS ',2D16.8 /
     *          ' RESIDUALS ARE        ',3D16.8 )
          ENDIF
      ENDIF
      RETURN
      END
