C
C   DRIVER FOR TESTING CMLIB ROUTINES
C     SGEEV   CGEEV
C     SSIEV   CHIEV
C     SSPEV
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(1)
      CALL XERMAX(1000)
C   TEST SGEEV CGEEV
      CALL EISQX1(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C   TEST SSIEV CHIEV SSPEV
      CALL EISQX2(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY,
     1 LICEPAK HAS FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- SUBLIBRARY LICEPACK PASSED ALL TESTS ----- ')
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      SUBROUTINE EISQX1(KPRINT,IPASS)
C
C     THIS QUICK CHECK ROUTINE IS WRITTEN FOR EISPACK DRIVERS
C     SGEEV AND CGEEV.  THE EIGENVALUES OF INPUT MATRIX A(.,.)
C     ARE STORED IN EK(.).  RELERR IS THE RELATIVE ACCURACY
C     REQUIRED FOR THEM TO PASS.
C
      COMMON /MSG/ ICNT,ITEST(38)
      COMMON /UNIT/ LUN
      INTEGER KPRINT,IPASS,ICNT,ITEST,LUN
      INTEGER LDA,N,LDV,JOB,I,J,ID
      REAL A(3,3),EK(3),W(9)
      REAL ERR,ERRI,RELERR,RECJ
      COMPLEX AC(3,3),EC(3),VC(3,3)
      DATA LDA,N,LDV / 3*3 /
      DATA A / 1., -2., 6., -1., 0., -3., 2., 5., 6. /
      DATA EK / -1., 3., 5. /
      IPASS = 1
      RELERR = SQRT(R2MACH(4))
      DO 20 J=1,N
         DO 10 I=1,N
            AC(I,J) = CMPLX(A(I,J),0.)
   10       CONTINUE
   20    CONTINUE
      JOB = 1
      CALL CGEEV(AC,LDA,N,EC,VC,LDV,W,JOB,INFO)
      IF (INFO .NE. 0) CALL XERROR(' EISQX1 -- INFO .NE. 0',22,101,2)
      DO 40 J=1,N
         ERR = ABS(AIMAG(EC(J)))
         IF (ERR .GE. RELERR) IPASS = 0
         RECJ = REAL(EC(J))
         ERR = ABS(RECJ - EK(1))
         ID = 1
         DO 30 I=2,N
            ERRI = ABS(RECJ - EK(I))
            IF (ERRI .LT. ERR) ID = I
            ERR = AMIN1(ERRI,ERR)
   30       CONTINUE
         IF (ABS(RECJ-EK(ID))/ABS(EK(ID)) .GE. RELERR) IPASS = 0
   40    CONTINUE
      JOB = 0
      CALL SGEEV(A,LDA,N,EC,VC,LDV,W,JOB,INFO)
      IF (INFO .NE. 0) CALL XERROR(' EISQX1 -- INFO .NE. 0',22,102,2)
      DO 60 J=1,N
         ERR = ABS(AIMAG(EC(J)))
         IF (ERR .GE. RELERR) IPASS = 0
         RECJ = REAL(EC(J))
         ERR = ABS(RECJ - EK(1))
         ID = 1
         DO 50 I=2,N
            ERRI = ABS(RECJ - EK(I))
            IF (ERRI .LT. ERR) ID = I
            ERR = AMIN1(ERRI,ERR)
   50       CONTINUE
         IF (ABS(RECJ-EK(ID))/ABS(EK(ID)) .GE. RELERR) IPASS = 0
   60    CONTINUE
      IF (KPRINT.GE.2 .AND. IPASS.NE.0) WRITE (LUN,670)
  670 FORMAT('EISQX1 PASSES ALL TESTS.')
      IF (KPRINT.GE.1 .AND. IPASS.EQ.0) WRITE (LUN,680)
  680 FORMAT('EISQX1 FAILS SOME TESTS.')
      RETURN
      END
      SUBROUTINE EISQX2(KPRINT,IPASS)
C
C     THIS QUICK CHECK ROUTINE IS WRITTEN FOR EISPACK DRIVERS
C     SSIEV, CHIEV AND SSPEV.  THE EIGENVALUES OF INPUT MATRIX
C     A(.,.) ARE STORED IN EK(.).  RELERR IS THE RELATIVE
C     ACCURACY REQUIRED FOR THEM TO PASS.
C
      COMMON /MSG/ ICNT,ITEST(38)
      COMMON /UNIT/ LUN
      INTEGER KPRINT,IPASS,ICNT,ITEST,LUN
      INTEGER LDA,N,LDV,JOB,I,J,ID
      REAL A(4,4),AP(10),E(4),V(4,4),EK(4),W(16)
      REAL ERR,ERRI,RELERR
      COMPLEX AC(4,4),VC(4,4)
      EQUIVALENCE (V,VC)
      DATA LDA,N,LDV / 3*4 /
      DATA AP / 5., 4., 5., 1., 1., 4., 1., 1., 2., 4. /
      DATA EK / 1., 2., 5., 10. /
      IPASS = 1
      RELERR = SQRT(R2MACH(4))
      ID = 0
      DO 20 J=1,N
         DO 10 I=1,J
            ID = ID + 1
            A(I,J) = AP(ID)
            AC(I,J) = CMPLX(AP(ID),0.)
   10       CONTINUE
   20    CONTINUE
      JOB = 1
      CALL CHIEV(AC,LDA,N,E,VC,LDV,W,JOB,INFO)
      IF (INFO .NE. 0) CALL XERROR(' EISQX2 -- INFO .NE. 0',22,101,2)
      DO 40 J=1,N
         ERR = ABS(E(J) - EK(1))
         ID = 1
         DO 30 I=2,N
            ERRI = ABS(E(J) - EK(I))
            IF (ERRI .LT. ERR) ID = I
            ERR = AMIN1(ERRI,ERR)
   30       CONTINUE
         IF (ABS(E(J)-EK(ID))/ABS(EK(ID)) .GE. RELERR) IPASS = 0
   40    CONTINUE
      CALL SSIEV(A,LDA,N,E,W,JOB,INFO)
      IF (INFO .NE. 0) CALL XERROR(' EISQX2 -- INFO .NE. 0',22,102,2)
      DO 60 J=1,N
         ERR = ABS(E(J) - EK(1))
         ID = 1
         DO 50 I=2,N
            ERRI = ABS(E(J) - EK(I))
            IF (ERRI .LT. ERR) ID = I
            ERR = AMIN1(ERRI,ERR)
   50       CONTINUE
         IF (ABS(E(J)-EK(ID))/ABS(EK(ID)) .GE. RELERR) IPASS = 0
   60    CONTINUE
      JOB = 0
      CALL SSPEV(AP,N,E,V,LDV,W,JOB,INFO)
      IF (INFO .NE. 0) CALL XERROR(' EISQX2 -- INFO .NE. 0',22,103,2)
      DO 80 J=1,N
         ERR = ABS(E(J) - EK(1))
         ID = 1
         DO 70 I=2,N
            ERRI = ABS(E(J) - EK(I))
            IF (ERRI .LT. ERR) ID = I
            ERR = AMIN1(ERRI,ERR)
   70       CONTINUE
         IF (ABS(E(J)-EK(ID))/ABS(EK(ID)) .GE. RELERR) IPASS = 0
   80    CONTINUE
      IF (KPRINT.GE.2 .AND. IPASS.NE.0) WRITE (LUN,684)
  684 FORMAT('EISQX2 PASSES ALL TESTS.')
      IF (KPRINT.GE.1 .AND. IPASS.EQ.0) WRITE (LUN,686)
  686 FORMAT('EISQX2 FAILS SOME TESTS.')
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
