C
C   DRIVER FOR TESTING CMLIB ROUTINES
C      SGEFS   SGEIR
C      DGEFS
C      CGEFS   CGEIR
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      INTEGER NERR,TERR
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XERMAX(1000)
      CALL XSETF(0)
      TERR = 0
      IF(KPRINT.EQ.3)WRITE(LUN,998)
      CALL SGEQC(NERR,KPRINT,LUN)
      TERR = TERR + NERR
      CALL DGEQC(NERR,KPRINT,LUN)
      TERR = TERR + NERR
      CALL CGEQC(NERR,KPRINT,LUN)
      TERR = TERR + NERR
      IF(KPRINT.GE.2)WRITE(LUN,999) TERR
C
      IF(TERR.NE.0)ITEST=0
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR LINDRIVES HAS,
     1 FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- LINDRIVES PASSED ALL TESTS ----- ')
998   FORMAT (/,'*****QUICK CHECK PROGRAM GETEST*****')
999   FORMAT(/,'*****GETEST DETECTED A TOTAL OF',I2,' ERRORS*****'/)
            END
      SUBROUTINE CGEQC(NERR,KPRINT,LUN)
C
C     QUICK CHECK OF CGEFS AND CGEIR SUBROUTINES
C
C     LET A*X=B BE A COMPLEX PRECISION LINEAR SYSTEM WHERE THE
C     MATRIX IS OF THE PROPER TYPE FOR THE LINPACK SUBROUTINES
C     BEING CALLED.  THE VALUES OF A AND B AND THE PRE-COMPUTED
C     VALUES OF BXEX (THE SOLUTION VECTOR) ARE ENTERED WITH DATA
C     STATEMENTS.  THE COMPUTED TEST RESULTS FOR X ARE COMPARED TO
C     THE STORED PRE-COMPUTED VALUES. FAILURE OF THE TEST OCCURS
C     WHEN THERE IS LESS THAN 80% AGREEMENT BETWEEN THE ABSOLUTE
C     VALUES.  THERE ARE 2 TESTS-  ONE FOR THE NORMAL CASE AND ONE
C     FOR THE SINGULAR CASE.  A MESSAGE IS PRINTED INDICATING
C     WHETHER EACH SUBROUTINE HAS PASSED OR FAILED FOR EACH CASE.
C
C     NO INPUT ARGUMENTS ARE REQUIRED.
C     ON RETURN, NERR (INTEGER TYPE) CONTAINS THE TOTAL COUNT OF
C     ALL FAILURES DETECTED.
C     THE ROUTINES LISTED ABOVE CALL LINPACK SUBROUTINES WHICH IN
C     TURN CALL BLAS ROUTINES.
C
C     MESSAGE STATEMENTS ASSUME THAT 4 CHARACTERS MAY BE ACCOMO-
C     DATED BY A SINGLE MACHINE WORD.
C
C     OCTOBER 29, 1980          NANCY JACOBSEN, GROUP C-3
C                               LOS ALAMOS SCIENTIFIC LABORATORY
C                               LOS ALAMOS, NM 87545
C
C
      COMPLEX A(3,3),ATEMP(5,3),B(3),BTEMP(3),BXEX(3),WORK(12),XA,XB
      REAL DELX
      INTEGER IWORK(3),LIST(2),KPROG,NERR,LDA,ITASK,IND,I,J,N,INDX
      DATA A/(2.,3.),(1.,1.),(1.,2.),
     1(2.,0.),(1.,-1.),(0.,0.),
     2(0.,0.),(2.,5.),(3.,2.)/
      DATA B/(-1.,1.),(-5.,4.),(-4.,7.)/
      DATA BXEX/(.21459E-01,.209012E+01),
     1(.261373E+01,-.162231E+01),(.785407E+00,.109871E+01)/
      DATA LIST/4HGEFS,4HGEIR/
      DELX(XA,XB)=ABS(REAL(XA-XB))+ABS(AIMAG(XA-XB))
      N=3
      LDA=5
      NERR = 0
C
      DO 50 KPROG = 1,2
C
C     FIRST TEST CASE - NORMAL
C
      ITASK=1
      INDX = 0
      DO 10 I=1,N
          BTEMP(I) = B(I)
10    CONTINUE
      DO 12 J=1,N
          DO 11 I=1,N
            ATEMP(I,J) = A(I,J)
11        CONTINUE
12      CONTINUE
      IF (KPROG.EQ.1)CALL CGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (KPROG.EQ.2)CALL CGEIR(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.LT.0) WRITE(LUN,200) LIST(KPROG), IND
      IF (IND.LT.0) NERR = NERR + 1
C
C     CALCULATE ERROR FOR FIRST TEST
C
C
      DO 30 I=1,N
      IF(DELX(BXEX(I),BTEMP(I)) .GT. .0001) INDX = INDX + 1
30    CONTINUE
      IF (INDX.EQ.0.AND.KPRINT.GE.2) WRITE(LUN,100) LIST(KPROG)
      IF (INDX.GE.1) WRITE(LUN,200) LIST(KPROG)
      IF (INDX.GE.1) NERR = NERR + 1
C
C     SECOND TEST CASE - SINGULAR MATRIX
C
      ITASK=1
      DO 15 I=1,N
          BTEMP(I) = B(I)
15    CONTINUE
      DO 35 J=1,N
          DO 25 I=1,N
            ATEMP(I,J) = A(I,J)
25        CONTINUE
35    CONTINUE
      DO 40 J=1,N
          ATEMP(1,J) = (0.E0,0.E0)
40    CONTINUE
      IF (KPROG.EQ.1) CALL CGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (KPROG.EQ.2) CALL CGEIR(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.EQ.-4.AND.KPRINT.GE.2) WRITE(LUN,302) LIST(KPROG)
      IF (IND.NE.-4) WRITE(LUN,300) LIST(KPROG), IND
      IF (IND.NE.-4) NERR = NERR + 1
C
100   FORMAT (/,5X,'C',A4,' NORMAL TEST PASSED')
200   FORMAT (/,5X,'C',A4,' TEST FAILED')
201   FORMAT (/,5X,'C',A4,' TEST FAILED IND=',I3)
300   FORMAT (/,5X,'C',A4,' SINGULAR TEST FAILED, IND =',I3)
302   FORMAT (/,5X,'C',A4,' SINGULAR TEST PASSED')
500   FORMAT (/,5X,'NERR=',I2)
50    CONTINUE
      IF(KPRINT.GT.2)WRITE(LUN,500) NERR
      RETURN
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      SUBROUTINE DGEQC(NERR,KPRINT,LUN)
C
C     QUICK CHECK OF SUBROUTINE DGEFS
C
C     LET A*X=B BE A DOUBLE PRECISION LINEAR SYSTEM WHERE THE
C     MATRIX IS OF THE PROPER TYPE FOR THE LINPACK SUBROUTINES
C     BEING CALLED.  THE VALUES OF A AND B AND THE PRE-COMPUTED
C     VALUES OF BXEX (THE SOLUTION VECTOR) ARE ENTERED WITH DATA
C     STATEMENTS.  THE COMPUTED TEST RESULTS FOR X ARE COMPARED TO
C     THE STORED PRE-COMPUTED VALUES. FAILURE OF THE TEST OCCURS
C     WHEN THERE IS LESS THAN 80% AGREEMENT BETWEEN THE ABSOLUTE
C     VALUES.  THERE ARE 2 TESTS-  ONE FOR THE NORMAL CASE AND ONE
C     FOR THE SINGULAR CASE.  A MESSAGE IS PRINTED INDICATING
C     WHETHER EACH SUBROUTINE HAS PASSED OR FAILED FOR EACH CASE.
C
C     NO INPUT ARGUMENTS ARE REQUIRED.
C     ON RETURN, NERR (INTEGER TYPE) CONTAINS THE TOTAL COUNT OF
C     ALL FAILURES DETECTED.
C     THE ROUTINES LISTED ABOVE CALL LINPACK SUBROUTINES WHICH IN
C     TURN CALL BLAS ROUTINES.
C
C     MESSAGE STATEMENTS ASSUME THAT 4 CHARACTERS MAY BE ACCOMO-
C     DATED BY A SINGLE MACHINE WORD.
C
C     OCTOBER 22, 1980          NANCY JACOBSEN, GROUP C-3
C                               LOS ALAMOS SCIENTIFIC LABORATORY
C                               LOS ALAMOS, NM 87545
C
C
      DOUBLE PRECISION A(5,4),ATEMP(5,4),B(4),BTEMP(4),BXEX(4),WORK(20),
     1                 D2MACH
      REAL ERRMX1,ERRCMP,ERRVAL
      INTEGER IWORK(4),LIST(2),KPROG,NERR,LDA,ITASK,IND,I,J,N
      DATA A/5.0D0,1.0D0,0.3D0,2.1D0,0.0D0,
     1 -1.0D0,-0.5D0,1.0D0,1.0D0,0.0D0,
     2 4.5D0,-1.0D0,-1.7D0,2.0D0,0.0D0,
     3 0.5D0,2.0D0,0.6D0,1.3D0,0.0D0/
      DATA B/0.0D0,3.5D0,3.6D0,2.4D0/
      DATA BXEX/.10D+01,.10D+01,-.10D+01,.10D+01/
      DATA LIST/4HGEFS,4HGEIR/
      N=4
      LDA=5
      NERR = 0
      ERRCMP = D2MACH(4)**0.8
C
      KPROG = 1
C
C     FIRST TEST CASE - NORMAL
C
      ITASK=1
      DO 10 I=1,N
          BTEMP(I) = B(I)
10    CONTINUE
      DO 12 J=1,N
          DO 11 I=1,N
            ATEMP(I,J) = A(I,J)
11        CONTINUE
12      CONTINUE
      IF (KPROG.EQ.1)CALL DGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.LT.0) WRITE(LUN,200) LIST(KPROG), IND
      IF (IND.LT.0) NERR = NERR + 1
C
C     CALCULATE ERROR FOR FIRST TEST
C
      ERRMX1 = 0.0
C
      DO 30 I=1,N
          ERRVAL = DABS (BTEMP(I)-BXEX(I))
          ERRMX1 = AMAX1  (ERRVAL,ERRMX1)
30    CONTINUE
      IF (ERRCMP.GT.ERRMX1.AND.KPRINT.GE.2) WRITE(LUN,100) LIST(KPROG)
      IF (ERRCMP.LT.ERRMX1) WRITE(LUN,200) LIST(KPROG), ERRMX1
      IF (ERRCMP.LT.ERRMX1) NERR = NERR + 1
C
C     SECOND TEST CASE - SINGULAR MATRIX
C
      ITASK=1
      DO 15 I=1,N
          BTEMP(I) = B(I)
15    CONTINUE
      DO 35 J=1,N
          DO 25 I=1,N
            ATEMP(I,J) = A(I,J)
25        CONTINUE
35    CONTINUE
      DO 40 J=1,N
          ATEMP(1,J) = 0.0E0
40    CONTINUE
      IF (KPROG.EQ.1) CALL DGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.EQ.-4.AND.KPRINT.GE.2)WRITE(LUN,302) LIST(KPROG)
      IF (IND.NE.-4) WRITE(LUN,300) LIST(KPROG), IND
      IF (IND.NE.-4) NERR = NERR + 1
C
100   FORMAT (/,5X,'D',A4,' NORMAL TEST PASSED')
200   FORMAT (/,5X,'D',A4,' TEST FAILED MAX ABS(ERROR) IS',E13.5)
201   FORMAT (/,5X,'D',A4,' TEST FAILED IND=',I3)
300   FORMAT (/,5X,'D',A4,' SINGULAR TEST FAILED, IND=',I3)
302   FORMAT (/,5X,'D',A4,' SINGULAR TEST PASSED')
500   FORMAT (/,5X,'NERR=',I2)
      IF(KPRINT.GT.2)WRITE(LUN,500) NERR
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE SGEQC(NERR,KPRINT,LUN)
C
C     QUICK CHECK OF SGEFS AND SGEIR SUBROUTINES
C
C     LET A*X=B BE A SINGLE PRECISION LINEAR SYSTEM WHERE THE
C     MATRIX IS OF THE PROPER TYPE FOR THE LINPACK SUBROUTINES
C     BEING CALLED.  THE VALUES OF A AND B AND THE PRE-COMPUTED
C     VALUES OF BXEX (THE SOLUTION VECTOR) ARE ENTERED WITH DATA
C     STATEMENTS.  THE COMPUTED TEST RESULTS FOR X ARE COMPARED TO
C     THE STORED PRE-COMPUTED VALUES. FAILURE OF THE TEST OCCURS
C     WHEN THERE IS LESS THAN 80% AGREEMENT BETWEEN THE ABSOLUTE
C     VALUES.  THERE ARE 2 TESTS-  ONE FOR THE NORMAL CASE AND ONE
C     FOR THE SINGULAR CASE.  A MESSAGE IS PRINTED INDICATING
C     WHETHER EACH SUBROUTINE HAS PASSED OR FAILED FOR EACH CASE.
C
C     NO INPUT ARGUMENTS ARE REQUIRED.
C     ON RETURN, NERR (INTEGER TYPE) CONTAINS THE TOTAL COUNT OF
C     ALL FAILURES DETECTED.
C     THE ROUTINES LISTED ABOVE CALL LINPACK SUBROUTINES WHICH IN
C     TURN CALL BLAS ROUTINES.
C
C     MESSAGE STATEMENTS ASSUME THAT 4 CHARACTERS MAY BE ACCOMO-
C     DATED BY A SINGLE MACHINE WORD.
C
C     OCTOBER 22, 1980          NANCY JACOBSEN, GROUP C-3
C                               LOS ALAMOS SCIENTIFIC LABORATORY
C                               LOS ALAMOS, NM 87545
C
C
      REAL A(5,4),ATEMP(5,4),B(4),BTEMP(4),BXEX(4),WORK(20)
      REAL ERRMX1,ERRCMP,ERRVAL
      INTEGER IWORK(4),LIST(2),KPROG,NERR,LDA,ITASK,IND,I,J,N
      DATA A/5.0,1.0,0.3,2.1,0.0,
     1 -1.0,-0.5,1.0,1.0,0.0,
     2 4.5,-1.0,-1.7,2.0,0.0,
     3 0.5,2.0,0.6,1.3,0.0/
      DATA B/0.0,3.5,3.6,2.4/
      DATA BXEX/.10E+01,.10E+01,-.10E+01,.10E+01/
      DATA LIST/4HGEFS,4HGEIR/
      N=4
      LDA=5
      NERR = 0
      ERRCMP = R2MACH(4)**0.8
C
      DO 50 KPROG = 1,2
C
C     FIRST TEST CASE - NORMAL
C
      ITASK=1
      DO 10 I=1,N
          BTEMP(I) = B(I)
10    CONTINUE
      DO 12 J=1,N
          DO 11 I=1,N
            ATEMP(I,J) = A(I,J)
11        CONTINUE
12      CONTINUE
      IF (KPROG.EQ.1)CALL SGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (KPROG.EQ.2)CALL SGEIR(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.LT.0) WRITE(LUN,200) LIST(KPROG), IND
      IF (IND.LT.0) NERR = NERR + 1
C
C     CALCULATE ERROR FOR FIRST TEST
C
      ERRMX1 = 0.0
C
      DO 30 I=1,N
          ERRVAL = ABS (BTEMP(I)-BXEX(I))
          ERRMX1 = AMAX1  (ERRVAL,ERRMX1)
30    CONTINUE
      IF (ERRCMP.GT.ERRMX1.AND.KPRINT.GE.2) WRITE(LUN,100) LIST(KPROG)
      IF (ERRCMP.LT.ERRMX1) WRITE(LUN,200) LIST(KPROG), ERRMX1
      IF (ERRCMP.LT.ERRMX1) NERR = NERR + 1
C
C     SECOND TEST CASE - SINGULAR MATRIX
C
      ITASK=1
      DO 15 I=1,N
          BTEMP(I) = B(I)
15    CONTINUE
      DO 35 J=1,N
          DO 25 I=1,N
            ATEMP(I,J) = A(I,J)
25        CONTINUE
35    CONTINUE
      DO 40 J=1,N
          ATEMP(1,J) = 0.0E0
40    CONTINUE
      IF (KPROG.EQ.1) CALL SGEFS(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (KPROG.EQ.2) CALL SGEIR(ATEMP,LDA,N,BTEMP,ITASK,IND,WORK,IWORK)
      IF (IND.EQ.-4.AND.KPRINT.GE.2) WRITE(LUN,302) LIST(KPROG)
      IF (IND.NE.-4) WRITE(LUN,300) LIST(KPROG), IND
      IF (IND.NE.-4) NERR = NERR + 1
C
100   FORMAT (/,5X,'S',A4,' NORMAL TEST PASSED')
200   FORMAT (/,5X,'S',A4,' TEST FAILED MAX ABS(ERROR) IS',E13.5)
201   FORMAT (/,5X,'S',A4,' TEST FAILED IND=',I3)
300   FORMAT (/,5X,'S',A4,' SINGULAR TEST FAILED, IND=',I3)
302   FORMAT (/,5X,'S',A4,' SINGULAR TEST PASSED')
500   FORMAT (/,5X,'NERR=',I2)
50    CONTINUE
      IF(KPRINT.GT.2.OR.NERR.NE.0)WRITE(LUN,500) NERR
      RETURN
      END
