C
C   DRIVER FOR TESTING CMLIB ROUTINES
C     R1MACH  D1MACH
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(1)
      CALL XERMAX(1000)
C   TEST R1MACH  D1MACH
      CALL MACHQX(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY MACHCON
     1  HAS FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- SUBLIBRARY MACHCON PASSED ALL TESTS ----- ')
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE MACHQX(KPRINT,IPASS)
C
C  MAIN PROGRAM TO TEST PORT CONSTANTS
C
C  WRITTEN BY:
C  JANICE M. KNAPP
C  CENTER FOR APPLIED MATHEMATICS
C  OCTOBER 15, 1979
C
      COMMON/UNIT/LUN
      REAL EPS, PEPS, S
      DOUBLE PRECISION DEPS, DPEPS, DS, D2MACH
      INTEGER KPRINT,IPASS
C
C  CALCULATE MACHINE EPSILON, CALL FOR THE SUBROUTINE-SUPPLIED VALUE,
C  AND COMPARE THE TWO.
C
C  FIRST EXECUTABLE STATEMENT
      IPASS=1
      EPS = 1.0
   10 EPS = EPS / 2.0
      S = 1.0 + EPS
      IF (S .GT. 1.0) GO TO 10
      EPS = 2.0 * EPS
C
      PEPS = R2MACH(4)
C
      IF (ABS (EPS - PEPS) .LT. 4 * PEPS) then
      IF(KPRINT.GE.2) WRITE(LUN,18)
      else
      IF(KPRINT.EQ.1) WRITE(LUN,19)
      IF(KPRINT.GE.2) WRITE (LUN, 20) PEPS, EPS
      IPASS=2
   18  FORMAT (1X,' ** TEST FOR R1MACH SUBROUTINE PASSED ** ')
   19  FORMAT (1X,' ** TEST FOR R1MACH SUBROUTINE FAILED ** ')
   20 FORMAT ('0', ' MACHINE EPSILON IN ERROR.',
     *        ' PORT VALUE OF MACHINE EPSILON =', E20.15 /
     *        27X, ' CALCULATED VALUE', 14X, '=', E20.15)
C
      endif
   30 DEPS = 1.0D0
   40 DEPS = DEPS / 2.0D0
      DS = 1.0D0 + DEPS
      IF (DS .GT. 1.0D0) GO TO 40
      DEPS = 2.0D0 * DEPS
C
      DPEPS = D2MACH(4)
C
      IF (ABS (DEPS - DPEPS) .LT. 4 * DPEPS) then
      IF(KPRINT.GE.2) WRITE(LUN,48)
      else
C
      IF(KPRINT.EQ.1) WRITE(LUN,49)
      IF(KPRINT.GE.2) WRITE (LUN, 50) DPEPS, DEPS
   48 FORMAT (1X,' ** TEST FOR D1MACH SUBROUTINE PASSED ** ')
   49 FORMAT (1X,' ** TEST FOR D1MACH SUBROUTINE FAILED ** ')
   50 FORMAT ('0', ' DOUBLE PRECISION MACHINE EPSILON IN ERROR.',
     *        ' PORT VALUE OF DP MACHINE EPSILON =', E20.15 /
     *        44X, ' CALCULATED VALUE', 17X, '=', E20.15)
      endif
      RETURN
      END
