C
C   DRIVER FOR TESTING CMLIB ROUTINES
C      DQAG  DQAGI  DQAGP  DQAGS  DQAWC  DQAWF  DQAWO  DQAWS  DQNG
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(-1)
      CALL XERMAX(1000)
      IF(KPRINT.LE.2) CALL XERMAX(0)
C  TEST DOUBLE PRECISION QUADPACK ROUTINES
      CALL DQADCK(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY,
     1 QUADPKD FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- QUADPKD SUBLIBRARY PASSED ALL TESTS ----- ')
            END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE DQADCK(KPRINT,IPASS)
C***BEGIN PROLOGUE  DQADCK
C**  CALLING PROGRAM FOR QUICK CHECK ROUTINES CDQNG, CDQAG,
C    CDQAGS, CDQAGP, CDQAGI, CDQAWO, CDQAWF AND CDQAWS
C***AUTHOR           ELSIE M. CLARK
C                     NATIONAL BUREAU OF STANDARDS
C***DESCRIPTION
C..........................................................
C
C  THIS PROGRAM CALLS THE QUICK CHECK ROUTINES FOR QUADPACK
C  INTEGRATORS QDNG, QDAG, QDAGS, QDAGP, QDAGI, QDAWO, QDAWF
C  AND QDAWS
C  KPRINT SPECIFIES THE AMOUNT OF PRINTING TO BE DONE BY
C  EACH QUICK CHECK ROUTINE
C  IPASS IS OUTPUT FROM EACH QUICK CHECK ROUTINE AND IS 1
C  WHEN ALL THE TESTS IN THE QUICK CHECK ROUTINE PASSED, 0
C  OTHERWISE.
C  FLAG IS 1 WHEN ALL THE TESTS PASS FOR EACH QUICK
C  ROUTINE, 0 OTHERWISE.
C
C..........................................................
C***END PROLOGUE  DQADCK
C
      COMMON/MSG/ICNT,ITEST(38)
      COMMON/UNIT/LUN
      INTEGER IPASS,LUN,KPRINT,FLAG
C  LUN DETERMINES OUTPUT UNIT
      FLAG = 1
      IF(KPRINT.GT.2) WRITE (LUN,5)
    5 FORMAT(' ********',/,' NOTE- (QUADPACK DOUBLE PRECISION TESTS)',
     X /,' THIS ROUTINE TESTS VARIOUS QUADPACK OPTIONS AND PRINTS',
     X /,' SOME ERROR DIAGNOSTICS IN SUBROUTINE XERROR.  THERE ',/,
     X ' SHOULD BE 38 ABNORMAL RETURN MESSAGES.  THE LAST LINE OF ',
     X /,' OUTPUT SHOULD BE-ALL QUADPACK QUICK CHECK ROUTINES PASSED-',
     X /,' (IMMEDIATELY FOLLOWING THE LAST ABNORMAL RETURN MESSAGE).',
     X /,' ANY OTHER FINAL LINE INDICATES AN ERROR.',/,'********')
      CALL CDQNG(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 10
        IF(KPRINT.GE.1)WRITE(LUN,15)
   15   FORMAT(' SOME TEST(S) IN CDQNG FAILED ')
        FLAG = 0
   10 CONTINUE
      CALL CDQAG(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 20
        IF(KPRINT.GE.1)WRITE(LUN,25)
   25   FORMAT(' SOME TEST(S) IN CDQAG FAILED ')
        FLAG = 0
   20 CONTINUE
      CALL CDQAGS(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 30
        IF(KPRINT.GE.1)WRITE(LUN,35)
   35   FORMAT(' SOME TEST(S) IN CDQAGS FAILED ')
        FLAG = 0
   30 CONTINUE
      CALL CDQAGP(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 40
        IF(KPRINT.GE.1)WRITE(LUN,45)
   45   FORMAT(' SOME TEST(S) IN CDQAGP FAILED ')
        FLAG = 0
   40 CONTINUE
      CALL CDQAGI(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 50
        IF(KPRINT.GE.1)WRITE(LUN,55)
   55   FORMAT(' SOME TEST(S) IN CDQAGI FAILED ')
        FLAG = 0
   50 CONTINUE
      CALL CDQAWO(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 60
        IF(KPRINT.GE.1)WRITE(LUN,65)
   65   FORMAT(' SOME TEST(S) IN CDQAWO FAILED ')
        FLAG = 0
   60 CONTINUE
      CALL CDQAWF(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 70
        IF(KPRINT.GE.1)WRITE(LUN,75)
   75   FORMAT(' SOME TEST(S) IN CDQAWF FAILED ')
        FLAG = 0
   70 CONTINUE
      CALL CDQAWS(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 80
        IF(KPRINT.GE.1)WRITE(LUN,85)
   85   FORMAT(' SOME TEST(S) IN CDQAWS FAILED ')
        FLAG = 0
   80 CONTINUE
      CALL CDQAWC(LUN,KPRINT,IPASS)
      IF (IPASS.NE.0) GO TO 90
        IF(KPRINT.GE.1)WRITE(LUN,95)
   95   FORMAT(' SOME TEST(S) IN CDQAWC FAILED ')
        FLAG = 0
   90 CONTINUE
      IPASS=FLAG
      IF(KPRINT.GT.1.AND.IPASS.EQ.1) WRITE(LUN,104)
      IF(KPRINT.NE.0.AND.IPASS.EQ.0) WRITE(LUN,105)
  104 FORMAT(' *****  ALL DP QUADPACK ROUTINES PASSED  ***** ')
  105 FORMAT(' *****  SOME DP QUADPACK TESTS FAILED  ***** ')
      RETURN
      END
      SUBROUTINE CDQAG(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAG,DF1G,DF2G,DF3G,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,EPSREL,ERROR,
     *EXACT1,EXACT2,EXACT3,DF1G,DF2G,DF3G,PI,RESULT,UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KEY,KPRINT,LAST,LENW,LIMIT,
     *  NEVAL
      DIMENSION IERV(2),IWORK(100),WORK(400)
      EXTERNAL DF1G,DF2G,DF3G
      DATA PI/0.31415926535897932D+01/
      DATA EXACT1/0.1154700538379252D+01/
      DATA EXACT2/0.11780972450996172D+00/
      DATA EXACT3/0.1855802D+02/
C
C TEST ON IER = 0
C
      IPASS = 1
      LIMIT = 100
      LENW = LIMIT*4
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      KEY = 6
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      A = 0.0D+00
      B = 0.1D+01
      CALL DQAG(DF1G,A,B,EPSABS,EPSREL,KEY,RESULT,ABSERR,NEVAL,IER,
     *LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      ERROR = DABS(EXACT1-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.ABSERR.AND.ABSERR.LE.EPSREL*DABS
     *(EXACT1))IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      LIMIT = 1
      LENW = LIMIT*4
      B = PI*0.2D+01
       CALL DQAG(DF2G,A,B,EPSABS,EPSREL,KEY,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT2,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 1
C
      UFLOW = D2MACH(1)
      LIMIT = 100
      LENW = LIMIT*4
      CALL DQAG(DF2G,A,B,UFLOW,0.0D+00,KEY,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT2,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 3 OR 1
C
      B = 0.1D+01
      CALL DQAG(DF3G,A,B,EPSABS,EPSREL,1,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT3,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 6
C
      LENW = 1
      CALL DQAG(DF1G,A,B,EPSABS,EPSREL,KEY,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0.AND.LAST.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAGI(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAGI,DT0,DT1,DT2,DT3,DT4
C              DT5,DPRIN,DF0S,DF1S,DF2S,DF3S,DF4S,DF5S
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION ABSERR,BOUND,D2MACH,EPMACH,EPSABS,
     *  EPSREL,ERROR,EXACT0,EXACT1,EXACT2,EXACT3,EXACT4,
     *  OFLOW,RESULT,DT0,DT1,DT2,DT3,DT4,DT5,UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KO,KPRINT,LAST,LENW,LIMIT,NEVAL
      DIMENSION WORK(800),IWORK(200),IERV(4)
      EXTERNAL DT0,DT1,DT2,DT3,DT4,DT5
      DATA EXACT0/2.0D+00/,EXACT1/0.115470066904D1/
      DATA EXACT2/0.909864525656D-02/
      DATA EXACT3/0.31415926535897932D+01/
      DATA EXACT4/0.19984914554328673D+04/
C
C TEST ON IER = 0
C
      IPASS = 1
      LIMIT = 200
      LENW = LIMIT*4
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      BOUND = 0.0D+00
      INF = 1
      CALL DQAGI(DT0,BOUND,INF,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      ERROR = DABS(RESULT-EXACT0)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.0.AND.ERROR.LE.EPSREL*DABS(EXACT0))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      CALL DQAGI(DT1,BOUND,INF,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  1,4,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 4 OR 1
C
      UFLOW = D2MACH(1)
      A = 0.1D+00
      CALL DQAGI(DT2,BOUND,INF,UFLOW,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.4.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT2,RESULT,ABSERR,NEVAL,IERV,3)
C
C TEST ON IER = 3 OR 4 OR 1 OR 2
C
      A = 0.0D+00
      B = 0.5D+01
      CALL DQAGI(DT3,BOUND,INF,UFLOW,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 2
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.2) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT3,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 4 OR 3 OR 1 OR 0
C
      B = 0.1D+01
      CALL DQAGI(DT4,BOUND,INF,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 3
      IERV(3) = 1
      IERV(4) = 0
      IP = 0
      IF(IER.EQ.4.OR.IER.EQ.3.OR.IER.EQ.1.OR.IER.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,4,KPRINT,IP,EXACT4,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 5
C
      OFLOW = D2MACH(2)
      CALL DQAGI(DT5,BOUND,INF,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.5) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,5,KPRINT,IP,OFLOW,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 6
C
      CALL DQAGI(DT1,BOUND,INF,EPSABS,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0.AND.LAST.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAGP(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAGP,DF1P,DF2P,DF3P
C                     DF4P,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,EPSREL,ERROR,
     *  EXACT1,
     *  EXACT2,EXACT3,DF1P,DF2P,DF3P,DF4P,OFLOW,POINTS,P1,P2,RESULT,
     *  UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KO,KPRINT,LAST,LENIW,LENW,LIMIT,
     *  NEVAL,NPTS2
      DIMENSION IERV(4),IWORK(205),POINTS(5),WORK(405)
      EXTERNAL DF1P,DF2P,DF3P,DF4P
      DATA EXACT1/0.4285277667368085D+01/
      DATA EXACT2/0.909864525656D-2/
      DATA EXACT3/0.31415926535897932D+01/
      DATA P1/0.1428571428571428D+00/
      DATA P2/0.6666666666666667D+00/
C
C TEST ON IER = 0
C
      IPASS = 1
      NPTS2 = 4
      LIMIT = 100
      LENIW = LIMIT*2+NPTS2
      LENW = LIMIT*4+NPTS2
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL=DMAX1(DSQRT(EPMACH),0.1D-07)
      A = 0.0D+00
      B = 0.1D+01
      POINTS(1) = P1
      POINTS(2) = P2
      CALL DQAGP(DF1P,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      ERROR = DABS(RESULT-EXACT1)
      IERV(1) = IER
      IP=0
      IF(IER.EQ.0.AND.ERROR.LE.EPSREL*DABS(EXACT1)) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      LENIW = 10
      LENW = LENIW*2-NPTS2
      CALL DQAGP(DF1P,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2, 4, 1 OR 3
C
      NPTS2 = 3
      POINTS(1) = 0.1D+00
      LENIW = LIMIT*2+NPTS2
      LENW = LIMIT*4+NPTS2
      UFLOW = D2MACH(1)
      A = 0.1D+00
      CALL DQAGP(DF2P,A,B,NPTS2,POINTS,UFLOW,0.0D+00,RESULT,ABSERR,
     * NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 3
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.3) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT2,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 3 OR 4 OR 1 OR 2
C
      NPTS2 = 2
      LENIW = LIMIT*2+NPTS2
      LENW = LIMIT*4+NPTS2
      A = 0.0D+00
      B = 0.5D+01
      CALL DQAGP(DF3P,A,B,NPTS2,POINTS,UFLOW,0.0D+00,RESULT,ABSERR,
     * NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 2
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.2) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT3,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 5
C
      B = 0.1D+01
      CALL DQAGP(DF4P,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.5) IP = 1
      IF(IP.EQ.0) IPASS = 0
      OFLOW = D2MACH(2)
      CALL DPRIN(KO,5,KPRINT,IP,OFLOW,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 6
C
      NPTS2 = 5
      LENIW = LIMIT*2+NPTS2
      LENW = LIMIT*4+NPTS2
      POINTS(1) = P1
      POINTS(2) = P2
      POINTS(3) = 0.3D+01
      CALL DQAGP(DF1P,A,B,NPTS2,POINTS,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0.AND.LAST.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAGS(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAGS,DF0S,DF1S,DF2S,DF3S
C                      DF45,DF5S,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,
     *  EPSREL,ERROR,EXACT0,EXACT1,EXACT2,EXACT3,EXACT4,
     *  DF0S,DF1S,DF2S,DF3S,DF4S,DF5S,OFLOW,RESULT,UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KPRINT,LAST,LENW,LIMIT,NEVAL
      DIMENSION IERV(4),IWORK(200),WORK(800)
      EXTERNAL DF0S,DF1S,DF2S,DF3S,DF4S,DF5S
      DATA EXACT0/0.2D+01/
      DATA EXACT1/0.115470066904D+01/
      DATA EXACT2/0.909864525656D-02/
      DATA EXACT3/0.31415926535897932D+01/
      DATA EXACT4/0.19984914554328673D+04/
C
C TEST ON IER = 0
C
      IPASS = 1
      LIMIT = 200
      LENW = LIMIT*4
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      A = 0.0D+00
      B = 0.1D+01
      CALL DQAGS(DF0S,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      ERROR = DABS(RESULT-EXACT0)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.0.AND.ERROR.LE.EPSREL*DABS(EXACT0))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      CALL DQAGS(DF1S,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  1,4,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1)IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 4 OR 1
C
      UFLOW = D2MACH(1)
      A = 0.1D+00
      CALL DQAGS(DF2S,A,B,UFLOW,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.4.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT2,RESULT,ABSERR,NEVAL,IERV,3)
C
C TEST ON IER = 3 OR 4 OR 1 OR 2
C
      A = 0.0D+00
      B = 0.5D+01
      CALL DQAGS(DF3S,A,B,UFLOW,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 2
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.2) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT3,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 4 OR 3 OR 1 OR 0
C
      B = 0.1D+01
      CALL DQAGS(DF4S,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 3
      IERV(3) = 1
      IERV(4) = 0
      IP = 0
      IF(IER.EQ.4.OR.IER.EQ.3.OR.IER.EQ.1.OR.IER.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,4,KPRINT,IP,EXACT4,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 5
C
      OFLOW = D2MACH(2)
      CALL DQAGS(DF5S,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.5) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,5,KPRINT,IP,OFLOW,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 6
C
      CALL DQAGS(DF1S,A,B,EPSABS,0.0D+00,RESULT,ABSERR,NEVAL,IER,
     *  LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0.AND.LAST.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAWC(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAWC,DF0O,DPRIN
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,
     *  EPSREL,ERROR,EXACT0,EXACT1,DF0C,DF1C,C,
     *  RESULT,UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KPRINT,LAST,LENW,LIMIT,NEVAL
      DIMENSION WORK(800),IWORK(200),IERV(2)
      EXTERNAL DF0C,DF1C
      DATA EXACT0/-0.6284617285065624D+03/
      DATA EXACT1/0.1855802D+01/
C
C TEST ON IER = 0
C
      IPASS = 1
      C = 0.5D+00
      A = -1.0D+00
      B = 1.0D+00
      LIMIT = 200
      LENW = LIMIT*4
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      CALL DQAWC(DF0C,A,B,C,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      ERROR = DABS(EXACT0-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.ABSERR.AND.ABSERR.LE.EPSREL*DABS(EXACT0))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      CALL DQAWC(DF0C,A,B,C,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,1,4,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 1
C
      UFLOW = D2MACH(1)
      CALL DQAWC(DF0C,A,B,C,UFLOW,0.0D+00,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 3 OR 1
C
      CALL DQAWC(DF1C,0.0D+00,B,C,UFLOW,0.0D+00,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 6
C
      EPSABS = 0.0D+00
      EPSREL = 0.0D+00
      CALL DQAWC(DF0C,A,B,C,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAWF(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAWF,DF0F,DF1F,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,D2MACH,EPSABS,EPMACH,
     *  ERROR,EXACT0,DF0F,DF1F,OMEGA,PI,RESULT,UFLOW,WORK
      INTEGER IER,IP,IPASS,KPRINT,LENW,LIMIT,LIMLST,LST,NEVAL
      DIMENSION IERV(4),IWORK(450),WORK(1425)
      EXTERNAL DF0F,DF1F
      DATA EXACT0/0.1422552162575912D+01/
      DATA PI/0.31415926535897932D+01/
C
C TEST ON IER = 0
C
      IPASS = 1
      MAXP1 = 21
      LIMLST = 50
      LIMIT = 200
      LENIW = LIMIT*2+LIMLST
      LENW = LENIW*2+MAXP1*25
      EPMACH = D2MACH(4)
      EPSABS = DMAX1(DSQRT(EPMACH),0.1D-02)
      A = 0.0D+00
      OMEGA = 0.8D+01
      INTEGR = 2
      CALL DQAWF(DF0F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,
     *  IER,LIMLST,LST,LENIW,MAXP1,LENW,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      ERROR = DABS(EXACT0-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.ABSERR.AND.ABSERR.LE.EPSABS)
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      LIMLST = 3
      LENIW = 403
      LENW = LENIW*2+MAXP1*25
      CALL DQAWF(DF0F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,
     *  IER,LIMLST,LST,LENIW,MAXP1,LENW,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 3 OR 4 OR 1 OR 2
C
      LIMLST = 50
      LENIW = LIMIT*2+LIMLST
      LENW = LENIW*2+MAXP1*25
      UFLOW = D2MACH(1)
      CALL DQAWF(DF1F,A,0.0D+00,1,UFLOW,RESULT,ABSERR,NEVAL,
     *  IER,LIMLST,LST,LENIW,MAXP1,LENW,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 2
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.2) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,PI,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 6
C
      LIMLST = 50
      LENIW = 20
      CALL DQAWF(DF0F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,
     *  IER,LIMLST,LST,LENIW,MAXP1,LENW,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 7
C
      LIMLST = 50
      LENIW = 52
      LENW = LENIW*2+MAXP1*25
      CALL DQAWF(DF0F,A,OMEGA,INTEGR,EPSABS,RESULT,ABSERR,NEVAL,
     *  IER,LIMLST,LST,LENIW,MAXP1,LENW,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.7) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,7,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAWO(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAWO,DF0O,DF1O,DF2O,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,EPMACH,EPSABS,
     *  EPSREL,ERROR,EXACT0,DF0O,DF1O,DF2O,
     *  OFLOW,OMEGA,PI,RESULT,D2MACH,UFLOW,WORK
      INTEGER IER,IERV,INTEGR,IP,IPASS,IWORK,KO,KPRINT,LAST,LENW,
     *  MAXP1,NEVAL
      DIMENSION WORK(1325),IWORK(400),IERV(4)
      EXTERNAL DF0O,DF1O,DF2O
      DATA EXACT0/0.1042872789432789D+05/
      DATA PI/0.31415926535897932D+01/
C
C TEST ON IER = 0
C
      IPASS = 1
      MAXP1 = 21
      LENIW = 400
      LENW = LENIW*2+MAXP1*25
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      A = 0.0D+00
      B = PI
      OMEGA = 0.1D+01
      INTEGR = 2
      CALL DQAWO(DF0O,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      ERROR = DABS(EXACT0-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.ABSERR.AND.ABSERR.LE.EPSREL*DABS(EXACT0))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      LENIW = 2
      LENW = LENIW*2+MAXP1*25
      CALL DQAWO(DF0O,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 4 OR 1
C
      UFLOW = D2MACH(1)
      LENIW = 400
      LENW = LENIW*2+MAXP1*25
      CALL DQAWO(DF0O,A,B,OMEGA,INTEGR,UFLOW,0.0D+00,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.4.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,3)
C
C TEST ON IER = 3 OR 4 OR 1 OR 2
C
      B = 0.5D+01
      OMEGA = 0.0D+00
      INTEGR = 1
      CALL DQAWO(DF1O,A,B,OMEGA,INTEGR,UFLOW,0.0D+00,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 4
      IERV(3) = 1
      IERV(4) = 2
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.4.OR.IER.EQ.1.OR.IER.EQ.2) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,PI,RESULT,ABSERR,NEVAL,IERV,4)
C
C TEST ON IER = 5
C
      B = 0.1D+01
      OFLOW = D2MACH(2)
      CALL DQAWO(DF2O,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.5) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,5,KPRINT,IP,OFLOW,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 6
C
      INTEGR = 3
      CALL DQAWO(DF0O,A,B,OMEGA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     * NEVAL,IER,LENIW,MAXP1,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0.AND.LAST.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQAWS(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED- D2MACH,DQAWS,DF0WS,DF1WS,DF2WS,DPRIN
C FOR FUTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,
     *  EPSREL,ERROR,EXACT0,EXACT1,DF0WS,DF1WS,ALFA,BETA,
     *  RESULT,UFLOW,WORK
      INTEGER IER,IP,IPASS,IWORK,KPRINT,LAST,LENW,LIMIT,NEVAL,INTEGR
      DIMENSION WORK(800),IWORK(200),IERV(2)
      EXTERNAL DF0WS,DF1WS
      DATA EXACT0/0.5350190569223644D+00/
      DATA EXACT1/0.1998491554328673D+04/
C
C TEST ON IER = 0
C
      IPASS = 1
      ALFA = -0.5D+00
      BETA = -0.5D+00
      INTEGR = 1
      A = 0.0D+00
      B = 0.1D+01
      LIMIT = 200
      LENW = LIMIT*4
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      CALL DQAWS(DF0WS,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      ERROR = DABS(EXACT0-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.EPSREL*DABS(EXACT0))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,0,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      CALL DQAWS(DF0WS,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,2,8,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,1,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
C
C TEST ON IER = 2 OR 1
C
      UFLOW = D2MACH(1)
      CALL DQAWS(DF0WS,A,B,ALFA,BETA,INTEGR,UFLOW,0.0D+00,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.2.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,2,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 3 OR 1
C
      CALL DQAWS(DF1WS,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IERV(2) = 1
      IP = 0
      IF(IER.EQ.3.OR.IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,3,KPRINT,IP,EXACT1,RESULT,ABSERR,NEVAL,IERV,2)
C
C TEST ON IER = 6
C
      INTEGR = 0
      CALL DQAWS(DF1WS,A,B,ALFA,BETA,INTEGR,EPSABS,EPSREL,RESULT,ABSERR,
     *  NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6) IP = 1
      IF(IP.EQ.0) IPASS = 0
      CALL DPRIN(KO,6,KPRINT,IP,EXACT0,RESULT,ABSERR,NEVAL,IERV,1)
      RETURN
      END
      SUBROUTINE CDQNG(KO,KPRINT,IPASS)
C
C SUBROUTINES OR FUNCTIONS NEEDED* D2MACH,DQNG,DF1N,DF2N,DPRIN
C
C FOR FURTHER DOCUMENTATION SEE ROUTINE CQPDOC
C
      DOUBLE PRECISION A,ABSERR,B,D2MACH,EPMACH,EPSABS,EPSREL,EXACT1,
     *  ERROR,EXACT2,DF1N,DF2N,RESULT,UFLOW
      INTEGER IER,IERV,IP,IPASS,KPRINT,NEVAL
      DIMENSION IERV(1)
      EXTERNAL DF1N,DF2N
      DATA EXACT1/0.7281029132255818D+00/
      DATA EXACT2/0.1D+02/
C
C TEST ON IER = 0
C
      IPASS = 1
      EPSABS = 0.0D+00
      EPMACH = D2MACH(4)
      UFLOW = D2MACH(1)
      EPSREL = DMAX1(DSQRT(EPMACH),0.1D-07)
      A=0.0D+00
      B=0.1D+01
      CALL DQNG(DF1N,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
      CALL DQNG(DF1N,A,B,EPSABS,EPSREL,RESULT,ABSERR,NEVAL,IER)
      IERV(1)=IER
      IP = 0
      ERROR = DABS(EXACT1-RESULT)
      IF(IER.EQ.0.AND.ERROR.LE.ABSERR.AND.ABSERR.LE.EPSREL*DABS(EXACT1))
     *  IP = 1
      IF(IP.EQ.0) IPASS = 0
      IF(KPRINT.NE.0) CALL DPRIN(KO,0,KPRINT,IP,EXACT1,RESULT,ABSERR,
     *  NEVAL,IERV,1)
C
C TEST ON IER = 1
C
      CALL DQNG(DF2N,A,B,UFLOW,0.0D+00,RESULT,ABSERR,NEVAL,IER)
      IERV(1) = IER
      IP=0
      IF(IER.EQ.1) IP = 1
      IF(IP.EQ.0) IPASS = 0
      IF(KPRINT.NE.0) CALL DPRIN(KO,1,KPRINT,IP,EXACT2,RESULT,ABSERR,
     *  NEVAL,IERV,1)
C
C TEST ON IER = 6
C
      EPSABS = 0.0D+00
      EPSREL = 0.0D+00
      CALL DQNG(DF1N,A,B,EPSABS,0.0D+00,RESULT,ABSERR,NEVAL,IER)
      IERV(1) = IER
      IP = 0
      IF(IER.EQ.6.AND.RESULT.EQ.0.0D+00.AND.ABSERR.EQ.0.0D+00.AND.
     *  NEVAL.EQ.0) IP = 1
      IF(IP.EQ.0) IPASS = 0
      IF(KPRINT.NE.0) CALL DPRIN(KO,6,KPRINT,IP,EXACT1,RESULT,ABSERR,
     *  NEVAL,IERV,1)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF0C(X)
      DOUBLE PRECISION X
      DF0C = 1.D0/(X*X+1.D-4)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF0F(X)
      DOUBLE PRECISION X
      DF0F = 0.0D+00
      IF(X.NE.0.0D+00) DF0F = DSIN(0.5D+02*X)/(X*DSQRT(X))
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF0O(X)
      DOUBLE PRECISION X
      DF0O = (0.2D+01*DSIN(X))**14
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF0S(X)
      DOUBLE PRECISION X
      DF0S = 0.0D+00
      IF(X.NE.0.0D+00) DF0S = 0.1D+01/DSQRT(X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF0WS(X)
      DOUBLE PRECISION X
      DF0WS  = DSIN(0.1D+02*X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1C(X)
      DOUBLE PRECISION X
      DF1C = 0.0D+00
      IF(X.NE.0.33D+00) DF1C = (X-0.5D+00)*DABS(X-0.33D+00)**(-0.9D+00)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1F(X)
      DOUBLE PRECISION X,X1,Y
      X1 = X+0.1D+01
      DF1F = 0.5D+01/X1/X1
      Y = 0.5D+01/X1
      IF(Y.GT.0.31415926535897932D+01) DF1F = 0.0D+00
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1G(X)
      DOUBLE PRECISION PI,X
      DATA PI/0.31415926535897932D+01/
      DF1G = 0.2D+01/(0.2D+01+DSIN(0.1D+02*PI*X))
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1N(X)
      DOUBLE PRECISION X
      DF1N=0.1D+01/(X**4+X**2+0.1D+01)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1O(X)
      DOUBLE PRECISION X
      DF1O = 0.1D+01
      IF(X.GT.0.31415926535897932D+01) DF1O = 0.0D+00
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1P(X)
      DOUBLE PRECISION ALFA1,ALFA2,P1,P2,X,D1,D2
C  P1 = 1/7, P2 = 2/3
      DATA P1/0.1428571428571428D+00/
      DATA P2/0.6666666666666667D+00/
      ALFA1 = -0.25D0
      ALFA2 = -0.5D0
      D1=DABS(X-P1)
      D2=DABS(X-P2)
      DF1P = 0.0D+00
      IF(D1.NE.0.0D+00.AND.D2.NE.0.0D+00) DF1P = D1**ALFA1+D2**ALFA2
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1S(X)
      DOUBLE PRECISION X
      DF1S = 0.2D+01/(0.2D+01+DSIN(0.314159D+02*X))
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF1WS(X)
      DOUBLE PRECISION X
      DF1WS = 0.00D+00
      IF(X-0.33D+00 .NE. 0.00D+00) DF1WS=DABS(X-0.33D+00)**(-0.999D+00)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF2G(X)
      DOUBLE PRECISION X
      DF2G = X*DSIN(0.3D+02*X)*DCOS(0.5D+02*X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF2N(X)
      DOUBLE PRECISION X
      DF2N=X**(-0.9D+00)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF2O(X)
      DOUBLE PRECISION X
      DF2O = 0.0D+00
      IF(X.NE.0.0D+00) DF2O = 0.1D+01/(X*X*DSQRT(X))
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF2P(X)
      DOUBLE PRECISION X
      DF2P = DSIN(0.314159D+03*X)/(0.314159D+01*X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF2S(X)
      DOUBLE PRECISION X
      DF2S = 0.1D+03
      IF(X.NE.0.0D+00) DF2S = DSIN(0.314159D+03*X)/(0.314159D+01*X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF3G(X)
      DOUBLE PRECISION X
      DF3G=DABS(X-0.33D+00)**(-.90D+00)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF3P(X)
      DOUBLE PRECISION X
      DF3P = 0.1D+01
      IF(X.GT.0.31415926535897932D+01) DF3P = 0.0D+00
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF3S(X)
      DOUBLE PRECISION X
      DF3S = 0.1D+01
      IF(X.GT.0.31415926535897932D+01) DF3S = 0.0D+00
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF4P(X)
      DOUBLE PRECISION X
      DF4P = 0.0D+00
      IF(X.GT.0.0D+00) DF4P = 0.1D+01/(X*DSQRT(X))
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF4S(X)
      DOUBLE PRECISION X
      DF4S = 0.00D+00
      IF(X-0.33D+00 .NE. 0.00D+00) DF4S=DABS(X-0.33D+00)**(-0.999D+00)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DF5S(X)
      DOUBLE PRECISION X
      DF5S = 0.0D+00
      IF(X.NE.0.0D+00) DF5S = 1.0D+00/(X*DSQRT(X))
      RETURN
      END
      SUBROUTINE DPRIN(KO,NUM1,KPRINT,IP,EXACT,RESULT,ABSERR,NEVAL,
     *  IERV,LIERV)
C
C***BEGIN PROLOGUE   DPRIN
C***DPRIN ROUTINE FOR QUADPACK QUICK CHECKS, 10.27.81
C***AUTHORS          ROBERT PIESSENS AND ELISE DE DONCKER
C                    APPL. MATH. AND PROGR. DIV.- K.U.LEUVEN
C***MODIFIED         ELSIE M. CLARK
C                    NATIONAL BUREAU OF STANDARDS
C***DESCRIPTION
C.....................................................................
C
C  THIS PROGRAM IS CALLED BY THE (DOUBLE PRECISION) QUADPACK QUICK
C  CHECK ROUTINES, FOR PRINTING OUT THEIR MESSAGES.
C
C.....................................................................
C***END PROLOGUE
C
      DOUBLE PRECISION ABSERR,ERROR,EXACT,RESULT
      INTEGER IER,IERV,IP,KO,KPRINT,LIERV,NEVAL,NUM1
      DIMENSION IERV(LIERV)
      IER = IERV(1)
      ERROR = DABS(EXACT-RESULT)
      IF(KPRINT-2) 10,20,40
10    IF(IP.EQ.1) GO TO 999
      GO TO 999
20    IF(IP.EQ.1) GO TO 30
      WRITE(KO,900) NUM1
      IF(NUM1.EQ.0) WRITE(KO,901)
      IF(NUM1.GT.0) WRITE(KO,902) NUM1
      IF(LIERV.GT.1) WRITE(KO,903) (IERV(K),K=2,LIERV)
      IF(NUM1.EQ.6) WRITE(KO,904)
      WRITE(KO,905)
      WRITE(KO,906)
      IF(NUM1.NE.5) WRITE(KO,907) EXACT,RESULT,ERROR,ABSERR,IER,NEVAL
      IF(NUM1.EQ.5) WRITE(KO,910) RESULT,ABSERR,IER,NEVAL
      GO TO 999
30    WRITE(KO,908) NUM1
      GO TO 999
40    IF(IP.EQ.1) WRITE(KO,908) NUM1
      IF(NUM1.EQ.0) WRITE(KO,901)
      IF(NUM1.GT.0) WRITE(KO,902) NUM1
      IF(LIERV.GT.1) WRITE(KO,903) (IERV(K),K=2,LIERV)
      IF(NUM1.EQ.6) WRITE(KO,904)
      WRITE(KO,905)
      WRITE(KO,906)
      IF(NUM1.NE.5) WRITE(KO,907) EXACT,RESULT,ERROR,ABSERR,IER,NEVAL
      IF(NUM1.EQ.5) WRITE(KO,910) RESULT,ABSERR,IER,NEVAL
900   FORMAT(' TEST ON IER = ',I1,' FAILED.')
901   FORMAT(' WE MUST HAVE IER = 0, ERROR.LE.ABSERR AND,
     *   ABSERR.LE.MAX(EPSABS,EPSREL*DABS(EXACT))')
902   FORMAT(' WE MUST HAVE IER = ',I1)
903   FORMAT(' OR IER =    ',8(I1,2X))
904   FORMAT(' RESULT, ABSERR, NEVAL AND EVENTUALLY LAST SHOULD,
     *   BE ZERO')
905   FORMAT(' WE HAVE  ')
906   FORMAT(' ',6X,'EXACT',11X,'RESULT',6X,'ERROR',4X,'ABSERR',
     *  4X,'IER     NEVAL'/
     *  ' ',42X,'(EST.ERR.)(FLAG)(NO F-EVAL)')
907   FORMAT(' ',2(D15.7,1X),2(D11.2,1X),I4,4X,I6)
908   FORMAT(' TEST ON IER = ',I2,' PASSED')
910   FORMAT(5X,'INFINITY',4X,D15.7,11X,D11.2,I5,4X,I6)
999   RETURN
      END
      DOUBLE PRECISION FUNCTION DT0(X)
      DOUBLE PRECISION A,B,DF0S,X,X1,Y
      A = 0.0D+00
      B = 0.1D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT0 = (B-A)*DF0S(Y)/X1/X1
      RETURN
      END
      DOUBLE PRECISION FUNCTION DT1(X)
      DOUBLE PRECISION A,B,DF1S,X,X1,Y
      A = 0.0D+00
      B = 0.1D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT1 = (B-A)*DF1S(Y)/X1/X1
      RETURN
      END
      DOUBLE PRECISION FUNCTION DT2(X)
      DOUBLE PRECISION A,B,DF2S,X,X1,Y
      A = 0.1D+00
      B = 0.1D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT2 = (B-A)*DF2S(Y)/X1/X1
      RETURN
      END
      DOUBLE PRECISION FUNCTION DT3(X)
      DOUBLE PRECISION A,B,DF3S,X,X1,Y
      A = 0.0D+00
      B = 0.5D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT3 = (B-A)*DF3S(Y)/X1/X1
      RETURN
      END
      DOUBLE PRECISION FUNCTION DT4(X)
      DOUBLE PRECISION A,B,DF4S,X,X1,Y
      A = 0.0D+00
      B = 0.1D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT4 = (B-A)*DF4S(Y)/X1/X1
      RETURN
      END
      DOUBLE PRECISION FUNCTION DT5(X)
      DOUBLE PRECISION A,B,DF5S,X,X1,Y
      A = 0.0D+00
      B = 0.1D+01
      X1 = X+0.1D+01
      Y = (B-A)/X1+A
      DT5 = (B-A)*DF5S(Y)/X1/X1
      RETURN
      END
