C
C   DRIVER FOR TESTING CMLIB ROUTINES
C     SLVBLK  DTBLOK
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(1)
      CALL XERMAX(1000)
C   TEST SLVBLK  DTBLOK
      CALL SLVBQX(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY SLVBLK
     1  HAS FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- SUBLIBRARY SLVBLK PASSED ALL TESTS ----- ')
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE SLVBQX(KPRINT,IPASS)
C  TEST PROGRAM FOR THE SOLVEBLOK PACKAGE SOLVES AN 11TH ORDER LINEAR
C  ALMOST BLOCK DIAGONAL SYSTEM.
      COMMON/UNIT/LUN
      INTEGER KPRINT,IPASS
      DIMENSION BLOKS(61),B(16),X(11), IPIVOT(16),INTEGS(15)
C                 NROW NCOL LAST
      DATA INTEGS/  3 ,  4 ,  2
     2           ,  3 ,  3 ,  3
     3           ,  3 ,  4 ,  1
     4           ,  3 ,  4 ,  1
     5           ,  4 ,  4 ,  4/
      DATA BLOKS /.1,.2,-1.,  2.,-.2,.3,  -.1,-.2,-.3,  -.1,4.,.3
     2           ,0.,-.4,3.,  0.,.4,.5,  0.,-5.,-.5
     3           ,.6,.5,3.,  -.6,4.,.4,  -.6,.5,-.4,  5.,-.5,.4
     4           ,0.,0.,.3,  0.,0.,-.3,  0.,0.,.3,  0.,0.,7.
     5           ,0.,0.,.2,6.,  0.,0.,-.2,.1,  0.,0.,-.2,-.1,
     5            0.,0.,8.,-.1/
      DATA B /1.94,3.04,-.83,  0.,-3.54,2.75,  1.32,2.35,1.96,
     2       2*0.,1.52,  2*0.,.78,2.40 /
      IPASS=1
      NBLOKS = 5
      N = 11
      CALL SLVBLK ( BLOKS, INTEGS, NBLOKS, B, IPIVOT, X, IFLAG )
      ERROR = 0.
      DO 10 I=1,N
         B(I) = FLOAT(12-I)/10. - X(I)
   10    ERROR = AMAX1(ERROR,ABS(B(I)))
      IF (ERROR.LT.50.*R2MACH(4)) THEN
       IF(KPRINT.GE.2) WRITE(LUN,888)
      ELSE
       IF(KPRINT.GE.2) THEN
       WRITE(LUN,889)IFLAG,ERROR,(IPIVOT(I),I=1,16),(I,X(I),B(I),I=1,N)
       IF(KPRINT.GE.1) WRITE(LUN,890)
       IPASS=2
       ENDIF
      ENDIF
  888 FORMAT (1X,/,' ** TEST FOR SUBROUTINE SLVBLK PASSED ** ')
  889 FORMAT (1X,/,' CHECKOUT SOLVEBLOK ROUTINES'/
     *         ' IFLAG =',I2,', MAXIMUM ERROR =',E9.4//
     *         ' IPIVOT =',5(2X,3I2),I2//
     *        '  I     X(I)       ERROR(I)'/(I3,F11.5,E13.5))
  890 FORMAT (1X,/,' ** TEST FOR SUBROUTINE SLVBLK FAILED ** ')
 
      CALL DTBLOK ( BLOKS, INTEGS, NBLOKS, IPIVOT, IFLAG,
     *              DETSGN, DETLOG )
      DET = DETSGN*EXP(DETLOG)
      ERROR = DET - 2.418821899E6
      EPS = MAX(1.0E-8,50.0*R2MACH(4))
      IF((ABS(ERROR)/DET).LT.EPS) THEN
       IF(KPRINT.GE.2) WRITE(LUN,891)
      ELSE
       IF(KPRINT.GE.2) WRITE(LUN,892) DET,ERROR
       IF(KPRINT.GE.1) WRITE(LUN,893)
       IPASS=2
  891 FORMAT (1X,/,' ** TEST FOR SUBROUTINE DTBLOK PASSED ** ')
  892 FORMAT(//' DETERMINANT =',E15.8,'   ERROR = ',E11.5)
  893 FORMAT (1X,/,' ** TEST FOR SUBROUTINE DTBLOK FAILED ** ')
      END IF
                                        RETURN
      END
