C
C   DRIVER FOR TESTING CMLIB PROGRAMS
C     ISMAX
C     ISAMIN
C     ISMIN
C
C    ONE INPUT DATA CARD IS REQUIRED
C         READ(LIN,1) KPRINT,TIMES
C    1    FORMAT(I1,E10.0)
C
C     KPRINT = 0   NO PRINTING
C              1   NO PRINTING FOR PASSED TESTS, SHORT MESSAGE
C                  FOR FAILED TESTS
C              2   PRINT SHORT MESSAGE FOR PASSED TESTS, FULLER
C                  INFORMATION FOR FAILED TESTS
C              3   PRINT COMPLETE QUICK-CHECK RESULTS
C
C                ***IMPORTANT NOTE***
C         ALL QUICK CHECKS USE ROUTINES R2MACH AND D2MACH
C         TO SET THE ERROR TOLERANCES.
C     TIMES IS A CONSTANT MULTIPLIER THAT CAN BE USED TO SCALE THE
C     VALUES OF R1MACH AND D1MACH SO THAT
C               R2MACH(I) = R1MACH(I) * TIMES   FOR I=3,4,5
C               D2MACH(I) = D1MACH(I) * TIMES   FOR I=3,4,5
C     THIS MAKES IT EASILY POSSIBLE TO CHANGE THE ERROR TOLERANCES
C     USED IN THE QUICK CHECKS.
C     IF TIMES .LE. 0.0 THEN TIMES IS DEFAULTED TO 1.0
C
C              ***END NOTE***
C
      COMMON/UNIT/LUN
      COMMON/MSG/ICNT,JTEST(38)
      COMMON/XXMULT/TIMES
      LUN=I1MACH(2)
      LIN=I1MACH(1)
      ITEST=1
C
C     READ KPRINT,TIMES PARAMETERS FROM DATA CARD..
C
      READ(LIN,1) KPRINT,TIMES
1     FORMAT(I1,E10.0)
      IF(TIMES.LE.0.) TIMES=1.
      CALL XSETUN(LUN)
      CALL XSETF(1)
      CALL XERMAX(1000)
C   TEST ISMIN ISAMIN ISMAX
      CALL XBLAQX(KPRINT,IPASS)
      ITEST=ITEST*IPASS
C
      IF(KPRINT.GE.1.AND.ITEST.NE.1) WRITE(LUN,2)
2     FORMAT(/' ***** WARNING -- AT LEAST ONE TEST FOR SUBLIBRARY XBLAS
     1HAS FAILED ***** ')
      IF(KPRINT.GE.1.AND.ITEST.EQ.1) WRITE(LUN,3)
3     FORMAT(/' ----- SUBLIBRARY XBLAS PASSED ALL TESTS ----- ')
      END
      DOUBLE PRECISION FUNCTION D2MACH(I)
      DOUBLE PRECISION D1MACH
      COMMON/XXMULT/TIMES
      D2MACH=D1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      D2MACH = D2MACH * DBLE(TIMES)
      RETURN
      END
      REAL FUNCTION R2MACH(I)
      COMMON/XXMULT/TIMES
      R2MACH=R1MACH(I)
      IF(I.EQ.1.OR. I.EQ.2) RETURN
      R2MACH = R2MACH * TIMES
      RETURN
      END
      SUBROUTINE XBLAQX(KPRINT,IPASS)
C     QUICK TEST FOR ISMAX, ISAMIN AND ISMIN
      COMMON/UNIT/LUN
      REAL X(5), A(10), B(10)
      COMPLEX CX(2),CY(2)
      INTEGER I, KPRINT, IPASS
      IPASS=1
      X(1)=-2.
      X(2)=-3.
      X(3)=0.
      X(4)=1.
      X(5)=4.
      IER=0
      IF(ISMAX(5,X,1) .NE. 5) IER=IER+1
      IF(KPRINT.GT.2) WRITE(LUN,*) 'IER= ',IER,' AFTER ISMAX CALL'
      IF(ISAMIN(5,X,1) .NE. 3) IER=IER+1
      IF(KPRINT.GT.2) WRITE(LUN,*) 'IER= ',IER,' AFTER ISAMIN CALL'
      IF(ISMIN(5,X,1) .NE. 2) IER=IER+1
      IF(KPRINT.GT.2) WRITE(LUN,*) 'IER= ',IER,' AFTER ISMIN CALL'
C      QUICK TEST FOR CSROT
      CX(1)=CMPLX(1.,0.)
      CX(2)=CMPLX(1.,0.)
      CY(1)=CMPLX(-1.,0.)
      CY(2)=CMPLX(-1.,0.)
      INC=1
      CALL CSROT(2,CX,INC,CY,INC,0.0,1.0)
      IF(REAL(CX(1)).NE.-1. .OR. AIMAG(CX(1)).NE.0. .OR.
     1 REAL(CX(2)).NE.-1. .OR. AIMAG(CX(2)).NE.0. .OR.
     2 REAL(CY(1)).NE.-1. .OR. AIMAG(CY(1)).NE.0. .OR.
     3 REAL(CY(2)).NE.-1. .OR. AIMAG(CY(2)).NE.0.) IER=IER+1
      IF(KPRINT.GT.2) WRITE(LUN,*) 'IER= ',IER,' AFTER CSROT CALL'
C     QUICK TEST FOR SCOPYM
      DO 10 I = 1,10
          A(I) = I
   10 CONTINUE
      CALL SCOPYM(10,A,1,B,1)
      DO 11 I = 1,10
      IF(A(I) + B(I) .NE. 0) IER = IER + 1
   11 CONTINUE
      IF(KPRINT.GT.2) WRITE(LUN,*) 'IER= ',IER,' AFTER LOOP'
      IF(IER.NE.0) THEN
        IPASS=2
        IF(KPRINT.GE.1) WRITE (LUN,100) IER
        IF(KPRINT.GE.1) WRITE(LUN,101)
      ELSE
        IPASS=1
        IF(KPRINT.GE.2) WRITE(LUN,102)
      ENDIF
  100 FORMAT (/' THERE WERE ',I3,'ERRORS IN XBLAS ROUTINES ')
  101 FORMAT (/' ** TEST FOR XBLAS ROUTINES FAILED ** ')
  102 FORMAT(/' ** TEST FOR XBLAS ROUTINES PASSED ** ')
      RETURN
      END
